from __future__ import annotations

import json
import textwrap

from django.db import models

from django.contrib.postgres.search import TrigramSimilarity


def json_format_with_wrap(item: [] | {} | str | None, width: int = 70) -> str:
    """Formatuje 'ładnie' dane JSON, w sytuacji, gdyby jakas linia
    przekraczała szerokośc ``width``, zawija.
    """
    s = json.dumps(item, indent=2).split("\n")
    news = []
    for elem in s:
        if len(elem) > width:
            elem = textwrap.fill(elem, width=width)
        news.append(elem)
    return "\n".join(news)


def perform_trigram_search(
    queryset,
    trigram_db_field,
    trigram_db_value,
    max_number_records=5,
    minimum_acceptable_similarity=0.6,
    db_trigram_target="podobienstwo",
    similarity_step_down=0.05,
    similarity_step_up=0.01,
    max_steps=20,
) -> list[models.Model] | None:
    """Performs a TrigramSimilarity search, starting from similarity value of
    1.0, desceding every ``step`` as long as the number of records is below ``max_number_records``

    :param queryset: Django queryset, can be anything.
    :param trigram_db_field: trigram database field, can be Lower('db_column') etc,
    :param trigram_db_value: value you're looking for in trigram_db_field, will be used
    to compare similarity.
    """

    kwargs_annotate = {
        db_trigram_target: TrigramSimilarity(trigram_db_field, trigram_db_value)
    }

    current_similarity = 1.0
    current_step = 0
    while (
        current_similarity > minimum_acceptable_similarity or current_step >= max_steps
    ):
        current_step += 1
        kwargs_filter = {db_trigram_target + "__gte": current_similarity}
        my_queryset = queryset.annotate(**kwargs_annotate).filter(**kwargs_filter)

        cnt = my_queryset[: max_number_records + 1].count()
        if cnt == max_number_records + 1:
            # Za dużo rekordów. Zwiększ nieznacznie pożądane podobieństwo i szukaj dalej.
            current_similarity += similarity_step_up
            continue

        if cnt == 0:
            # Za mało rekordów. Zmniejsz poszukiwane prawdopodobieństwo:
            current_similarity -= similarity_step_down
            continue

        return my_queryset.order_by(f"-{db_trigram_target}")



from django.conf.urls import include, url
from rest_framework import routers

from api_v1.viewsets.autor import (
    Autor_JednostkaViewSet,
    AutorViewSet,
    Funkcja_AutoraViewSet,
    TytulViewSet,
)
from api_v1.viewsets.nagroda import NagrodaViewSet
from api_v1.viewsets.openaccess import Czas_Udostepnienia_OpenAccess_ViewSet
from api_v1.viewsets.patent import Patent_AutorViewSet, PatentViewSet
from api_v1.viewsets.praca_doktorska import Praca_DoktorskaViewSet
from api_v1.viewsets.praca_habilitacyjna import Praca_HabilitacyjnaViewSet
from api_v1.viewsets.struktura import JednostkaViewSet, UczelniaViewSet, WydzialViewSet
from api_v1.viewsets.system import (
    Charakter_FormalnyViewSet,
    Dyscyplina_NaukowaViewSet,
    JezykViewSet,
    KonferencjaViewSet,
    Seria_WydawniczaViewSet,
    Typ_KBNViewSet,
)
from api_v1.viewsets.wydawca import Poziom_WydawcyViewSet, WydawcaViewSet
from api_v1.viewsets.wydawnictwo_ciagle import (
    Wydawnictwo_Ciagle_AutorViewSet,
    Wydawnictwo_Ciagle_StreszczenieViewSet,
    Wydawnictwo_Ciagle_Zewnetrzna_Baza_DanychViewSet,
    Wydawnictwo_CiagleViewSet,
)
from api_v1.viewsets.wydawnictwo_zwarte import (
    Wydawnictwo_Zwarte_AutorViewSet,
    Wydawnictwo_Zwarte_StreszczenieViewSet,
    Wydawnictwo_ZwarteViewSet,
)
from api_v1.viewsets.zrodlo import Rodzaj_ZrodlaViewSet, ZrodloViewSet

router = routers.DefaultRouter()

router.register(r"konferencja", KonferencjaViewSet)
router.register(r"seria_wydawnicza", Seria_WydawniczaViewSet)

router.register(r"czas_udostepnienia_openaccess", Czas_Udostepnienia_OpenAccess_ViewSet)

router.register(r"nagroda", NagrodaViewSet)
router.register(r"charakter_formalny", Charakter_FormalnyViewSet)
router.register(r"typ_kbn", Typ_KBNViewSet)
router.register(r"jezyk", JezykViewSet)
router.register(r"dyscyplina_naukowa", Dyscyplina_NaukowaViewSet)

router.register(r"poziom_wydawcy", Poziom_WydawcyViewSet)
router.register(r"wydawca", WydawcaViewSet)

router.register(r"wydawnictwo_zwarte", Wydawnictwo_ZwarteViewSet)
router.register(r"wydawnictwo_zwarte_autor", Wydawnictwo_Zwarte_AutorViewSet)
router.register(
    r"wydawnictwo_zwarte_streszczenie",
    Wydawnictwo_Zwarte_StreszczenieViewSet,
)

router.register(r"patent", PatentViewSet)
router.register(r"patent_autor", Patent_AutorViewSet)

router.register(r"wydawnictwo_ciagle", Wydawnictwo_CiagleViewSet)
router.register(r"wydawnictwo_ciagle_autor", Wydawnictwo_Ciagle_AutorViewSet)
router.register(
    r"wydawnictwo_ciagle_zewnetrzna_baza_danych",
    Wydawnictwo_Ciagle_Zewnetrzna_Baza_DanychViewSet,
)
router.register(
    r"wydawnictwo_ciagle_streszczenie",
    Wydawnictwo_Ciagle_StreszczenieViewSet,
)

router.register(r"praca_doktorska", Praca_DoktorskaViewSet)

router.register(r"praca_habilitacyjna", Praca_HabilitacyjnaViewSet)

router.register(r"rodzaj_zrodla", Rodzaj_ZrodlaViewSet)
router.register(r"zrodlo", ZrodloViewSet)

router.register(r"jednostka", JednostkaViewSet)
router.register(r"wydzial", WydzialViewSet)
router.register(r"uczelnia", UczelniaViewSet)

router.register(r"autor", AutorViewSet)
router.register(r"funkcja_autora", Funkcja_AutoraViewSet)
router.register(r"tytul", TytulViewSet)
router.register(r"autor_jednostka", Autor_JednostkaViewSet)

urlpatterns = [
    url(r"^", include(router.urls)),
    url(r"^api-auth/", include("rest_framework.urls", namespace="rest_framework")),
]



# Generated by Django 2.1.7 on 2019-03-24 17:28

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0155_CASCADE'),
    ]

    operations = [
        migrations.AddField(
            model_name='autor_dyscyplina',
            name='procent_dyscypliny',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True),
        ),
        migrations.AddField(
            model_name='autor_dyscyplina',
            name='procent_subdyscypliny',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True),
        ),
        migrations.AddField(
            model_name='autor_dyscyplina',
            name='subdyscyplina',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='subdyscyplina', to='bpp.Dyscyplina_Naukowa'),
        ),
        migrations.AlterField(
            model_name='autor_dyscyplina',
            name='dyscyplina',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='dyscyplina', to='bpp.Dyscyplina_Naukowa'),
        ),
    ]



from rest_framework import serializers

from api_v1.serializers.util import ChoicesSerializerField
from bpp.models import (
    Charakter_Formalny,
    Typ_KBN,
    Jezyk,
    Dyscyplina_Naukowa,
    Konferencja,
    Seria_Wydawnicza,
)


class Dyscyplina_NaukowaSerializer(serializers.ModelSerializer):
    class Meta:
        model = Dyscyplina_Naukowa
        fields = ["id", "kod", "nazwa", "widoczna"]


class Seria_WydawniczaSerializer(serializers.ModelSerializer):
    class Meta:
        model = Seria_Wydawnicza
        fields = ["id", "nazwa"]


class KonferencjaSerializer(serializers.ModelSerializer):
    typ_konferencji = ChoicesSerializerField()

    class Meta:
        model = Konferencja
        fields = [
            "id",
            "nazwa",
            "skrocona_nazwa",
            "rozpoczecie",
            "zakonczenie",
            "miasto",
            "panstwo",
            "baza_scopus",
            "baza_wos",
            "baza_inna",
            "typ_konferencji",
        ]


class Charakter_FormalnySerializer(serializers.HyperlinkedModelSerializer):
    charakter_pbn = serializers.StringRelatedField(read_only=True)
    rodzaj_pbn = ChoicesSerializerField()
    charakter_sloty = ChoicesSerializerField()

    class Meta:
        model = Charakter_Formalny
        fields = [
            "id",
            "nazwa",
            "skrot",
            "charakter_pbn",
            "rodzaj_pbn",
            "charakter_sloty",
        ]


class Typ_KBNSerializer(serializers.ModelSerializer):
    charakter_pbn = serializers.StringRelatedField()

    class Meta:
        model = Typ_KBN
        fields = ["id", "nazwa", "skrot", "artykul_pbn", "charakter_pbn"]


class JezykSerializer(serializers.ModelSerializer):
    class Meta:
        model = Jezyk
        fields = ["id", "nazwa", "skrot"]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-19 23:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0095_issn_dla_zwartego'),
    ]

    operations = [
        migrations.AlterField(
            model_name='autor',
            name='adnotacje',
            field=models.TextField(blank=True, db_index=True, default='', help_text='Pole do użytku wewnętrznego -\n        wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych\n        dla użytkowników końcowych.'),
        ),
        migrations.AlterField(
            model_name='bppuser',
            name='adnotacje',
            field=models.TextField(blank=True, db_index=True, default='', help_text='Pole do użytku wewnętrznego -\n        wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych\n        dla użytkowników końcowych.'),
        ),
        migrations.AlterField(
            model_name='jednostka',
            name='adnotacje',
            field=models.TextField(blank=True, db_index=True, default='', help_text='Pole do użytku wewnętrznego -\n        wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych\n        dla użytkowników końcowych.'),
        ),
        migrations.AlterField(
            model_name='konferencja',
            name='adnotacje',
            field=models.TextField(blank=True, db_index=True, default='', help_text='Pole do użytku wewnętrznego -\n        wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych\n        dla użytkowników końcowych.'),
        ),
        migrations.AlterField(
            model_name='patent',
            name='adnotacje',
            field=models.TextField(blank=True, db_index=True, default='', help_text='Pole do użytku wewnętrznego -\n        wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych\n        dla użytkowników końcowych.'),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='adnotacje',
            field=models.TextField(blank=True, db_index=True, default='', help_text='Pole do użytku wewnętrznego -\n        wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych\n        dla użytkowników końcowych.'),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='adnotacje',
            field=models.TextField(blank=True, db_index=True, default='', help_text='Pole do użytku wewnętrznego -\n        wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych\n        dla użytkowników końcowych.'),
        ),
        migrations.AlterField(
            model_name='uczelnia',
            name='adnotacje',
            field=models.TextField(blank=True, db_index=True, default='', help_text='Pole do użytku wewnętrznego -\n        wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych\n        dla użytkowników końcowych.'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='adnotacje',
            field=models.TextField(blank=True, db_index=True, default='', help_text='Pole do użytku wewnętrznego -\n        wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych\n        dla użytkowników końcowych.'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='adnotacje',
            field=models.TextField(blank=True, db_index=True, default='', help_text='Pole do użytku wewnętrznego -\n        wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych\n        dla użytkowników końcowych.'),
        ),
        migrations.AlterField(
            model_name='wydzial',
            name='adnotacje',
            field=models.TextField(blank=True, db_index=True, default='', help_text='Pole do użytku wewnętrznego -\n        wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych\n        dla użytkowników końcowych.'),
        ),
        migrations.AlterField(
            model_name='zrodlo',
            name='adnotacje',
            field=models.TextField(blank=True, db_index=True, default='', help_text='Pole do użytku wewnętrznego -\n        wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych\n        dla użytkowników końcowych.'),
        ),
    ]



from unittest.mock import Mock

import pytest
from django.urls import reverse

from bpp.admin.zglos_publikacje_helpers import (
    KorzystaZNumeruZgloszeniaInlineMixin,
    KorzystaZNumeruZgloszeniaMixin,
    UzupelniajWstepneDanePoNumerzeZgloszeniaMixin,
)
from bpp.const import NUMER_ZGLOSZENIA_PARAM


def test_KorzystaZNumeruZgloszeniaMixin(zgloszenie_publikacji, rf):
    req = rf.get("/", {NUMER_ZGLOSZENIA_PARAM: zgloszenie_publikacji.pk})
    assert (
        KorzystaZNumeruZgloszeniaMixin.get_zgloszenie_publikacji(None, req)
        == zgloszenie_publikacji
    )


def test_KorzystaZNumeruZgloszeniaInlineMixin(zgloszenie_publikacji, rf):

    req = rf.get("/", {NUMER_ZGLOSZENIA_PARAM: zgloszenie_publikacji.pk})
    assert KorzystaZNumeruZgloszeniaInlineMixin().get_extra(req) == 1

    zgloszenie_publikacji.zgloszenie_publikacji_autor_set.all().delete()
    req = rf.get("/", {NUMER_ZGLOSZENIA_PARAM: zgloszenie_publikacji.pk})
    assert KorzystaZNumeruZgloszeniaInlineMixin().get_extra(req) == 0


def test_WstepneDanePoNumerzeZgloszeniaMixin_get_changeform_initial_data(
    zgloszenie_publikacji, rf, rok
):
    req = rf.get("/", {NUMER_ZGLOSZENIA_PARAM: zgloszenie_publikacji.pk})
    ret = UzupelniajWstepneDanePoNumerzeZgloszeniaMixin().get_changeform_initial_data(
        req
    )
    assert str(ret["rok"]) == str(rok)


def test_WstepneDanePoNumerzeZgloszeniaMixin_get_formset_kwargs(
    zgloszenie_publikacji, rf, rok
):
    class baseModel_AutorInline:
        pass

    req = rf.get("/", {NUMER_ZGLOSZENIA_PARAM: zgloszenie_publikacji.pk})
    ret = UzupelniajWstepneDanePoNumerzeZgloszeniaMixin().get_formset_kwargs(
        request=req,
        obj=Mock(pk=None),
        inline=baseModel_AutorInline(),
        prefix="autorzy_set",
    )
    assert (
        ret["initial"][0]["autor"]
        == zgloszenie_publikacji.zgloszenie_publikacji_autor_set.first().autor.pk
    )


@pytest.mark.parametrize(["url"], [("wydawnictwo_ciagle",), ("wydawnictwo_zwarte",)])
def test_integracyjny_strona_admina(admin_app, zgloszenie_publikacji, url):
    url = (
        reverse(f"admin:bpp_{url}_add")
        + f"?{NUMER_ZGLOSZENIA_PARAM}={zgloszenie_publikacji.pk}"
    )
    page = admin_app.get(url)

    assert page.forms[1]["autorzy_set-0-autor"].value == str(
        zgloszenie_publikacji.zgloszenie_publikacji_autor_set.first().autor.pk
    )


@pytest.mark.parametrize(["url"], [("wydawnictwo_ciagle",), ("wydawnictwo_zwarte",)])
def test_integracyjny_admin_czy_oplaty_przechodza(
    admin_app, zgloszenie_publikacji_z_oplata, url
):
    url = (
        reverse(f"admin:bpp_{url}_add")
        + f"?{NUMER_ZGLOSZENIA_PARAM}={zgloszenie_publikacji_z_oplata.pk}"
    )
    page = admin_app.get(url)
    assert (
        page.forms[1]["opl_pub_amount"].value
        == str(zgloszenie_publikacji_z_oplata.opl_pub_amount) + ".00"
    )
    assert page.forms[1]["opl_pub_cost_free"].value == "false"


@pytest.mark.parametrize(["url"], [("wydawnictwo_ciagle",), ("wydawnictwo_zwarte",)])
def test_integracyjny_admin_czy_public_dostep_dnia_ustawiony(
    admin_app, zgloszenie_publikacji, url
):
    url = (
        reverse(f"admin:bpp_{url}_add")
        + f"?{NUMER_ZGLOSZENIA_PARAM}={zgloszenie_publikacji.pk}"
    )
    page = admin_app.get(url)
    assert page.forms[1]["public_dostep_dnia"].value != ""



# -*- encoding: utf-8 -*-

from django.core.management import BaseCommand

from bpp.models.abstract import strony_regex, BRAK_PAGINACJI
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle
from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte


class Command(BaseCommand):
    help = "Weryfikuje regex strony_regex dla pola 'Szczegoly' wydawnictw ciągłych i zwartych"

    def handle(self, *args, **options):
        for klass in Wydawnictwo_Ciagle, Wydawnictwo_Zwarte:
            q = klass.objects.exclude(szczegoly="").exclude(szczegoly=None).filter(rok__gte=2010)
        total = q.count()
        ret = []
        cnt = 0
        for elem in q.only("pk", "tytul_oryginalny", "szczegoly"):

            cnt = False
            for bp in BRAK_PAGINACJI:
                if elem.szczegoly.find(bp) >= 0:
                    cnt = True
                    break

            if cnt:
                continue

            parsed = strony_regex.search(elem.szczegoly)
            if parsed is None or parsed.groupdict()['poczatek'] is None:
                print(elem.pk, "|", elem.szczegoly, "|", elem.tytul_oryginalny)



from notifications.core import convert_obj_to_channel_name


class ChannelSubscriberMixin:
    """Mixin który trzyma listę zasubskrybowanych obiektów i wyrzuca ją
    w context_data celem zarenderowania (i zasubskrybowania tych obiektów)
    po stronie HTMLu"""

    _subscribed_to = None
    channels_template_variable_name = "extraChannels"

    def subscribe_to(self, obj):
        if self._subscribed_to is None:
            self._subscribed_to = []
        cn = convert_obj_to_channel_name(obj)
        self._subscribed_to.append(cn)
        return cn

    def get_context_data(self, **kwargs):
        context = super(ChannelSubscriberMixin, self).get_context_data(**kwargs)
        context[self.channels_template_variable_name] = self._subscribed_to
        return context


class ChannelSubscriberSingleObjectMixin(ChannelSubscriberMixin):
    """Mixin ChannelSubscriber współpracujący z SingleObjectMixin, to znaczy
    subskrybujący kanał dla obiektu zwracanego w get_object"""

    def get_object(self, *args, **kw):
        obj = super(ChannelSubscriberSingleObjectMixin, self).get_object(*args, **kw)
        self.subscribe_to(obj)
        return obj



import urllib

from django_filters.views import FilterMixin
from django_tables2 import RequestConfig, SingleTableMixin

from formdefaults.helpers import FormDefaultsMixin
from long_running.tasks import perform_generic_long_running_task
from long_running.views import (
    CreateLongRunningOperationView,
    LongRunningDetailsView,
    LongRunningOperationsView,
    LongRunningResultsView,
    LongRunningRouterView,
    RestartLongRunningOperationView,
)
from nowe_raporty.views import BaseRaportAuthMixin
from raport_slotow.filters import (
    RaportSlotowUczelniaBezJednostekIWydzialowFilter,
    RaportSlotowUczelniaFilter,
)
from raport_slotow.forms import UtworzRaportSlotowUczelniaForm
from raport_slotow.models.uczelnia import RaportSlotowUczelnia
from raport_slotow.tables import (
    RaportSlotowUczelniaBezJednostekIWydzialowTable,
    RaportSlotowUczelniaTable,
)
from raport_slotow.util import MyExportMixin

from django.contrib.auth.mixins import LoginRequiredMixin

from django.utils import timezone

from bpp.views.mixins import UczelniaSettingRequiredMixin

from django_bpp.version import VERSION


class ListaRaportSlotowUczelnia(
    BaseRaportAuthMixin, FormDefaultsMixin, LongRunningOperationsView
):
    uczelnia_attr = "pokazuj_raport_slotow_uczelnia"
    title = "Raport slotów - uczelnia"
    model = RaportSlotowUczelnia

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["title"] = self.title
        return context


class UtworzRaportSlotowUczelnia(
    UczelniaSettingRequiredMixin,
    FormDefaultsMixin,
    CreateLongRunningOperationView,
):
    template_name = "raport_slotow/index.html"
    form_class = UtworzRaportSlotowUczelniaForm
    uczelnia_attr = "pokazuj_raport_slotow_uczelnia"
    title = "Raport slotów - uczelnia"
    model = RaportSlotowUczelnia
    task = perform_generic_long_running_task

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["title"] = self.title
        return context


class RouterRaportuSlotowUczelnia(BaseRaportAuthMixin, LongRunningRouterView):
    uczelnia_attr = "pokazuj_raport_slotow_uczelnia"
    model = RaportSlotowUczelnia


class SzczegolyRaportSlotowUczelnia(BaseRaportAuthMixin, LongRunningDetailsView):
    # template_name = "raport_slotow/raport_slotow_uczelnia_szcz.html"
    uczelnia_attr = "pokazuj_raport_slotow_uczelnia"
    export_formats = ["html", "xlsx"]
    # filterset_class = RaportSlotowUczelniaFilter
    model = RaportSlotowUczelnia


class WygenerujPonownieRaportSlotowUczelnia(
    BaseRaportAuthMixin, RestartLongRunningOperationView
):
    uczelnia_attr = "pokazuj_raport_slotow_uczelnia"
    model = RaportSlotowUczelnia


class SzczegolyRaportSlotowUczelniaListaRekordow(
    BaseRaportAuthMixin,
    LoginRequiredMixin,
    MyExportMixin,
    SingleTableMixin,
    LongRunningResultsView,
    FilterMixin,
):
    template_name = "raport_slotow/raport_slotow_uczelnia.html"
    uczelnia_attr = "pokazuj_raport_slotow_uczelnia"
    export_formats = ["html", "xlsx"]
    filterset_class = RaportSlotowUczelniaFilter
    model = RaportSlotowUczelnia

    def get_table_class(self):
        if self.parent_object.dziel_na_jednostki_i_wydzialy:
            return RaportSlotowUczelniaTable
        return RaportSlotowUczelniaBezJednostekIWydzialowTable

    def get_filterset_class(self):
        if self.parent_object.dziel_na_jednostki_i_wydzialy:
            return RaportSlotowUczelniaFilter
        return RaportSlotowUczelniaBezJednostekIWydzialowFilter

    def get_table(self, **kwargs):
        table_class = self.get_table_class()
        table = table_class(
            data=self.get_table_data(),
            od_roku=self.parent_object.od_roku,
            do_roku=self.parent_object.do_roku,
            slot=self.parent_object.slot,
            **kwargs,
        )
        RequestConfig(
            self.request, paginate=self.get_table_pagination(table)
        ).configure(table)
        return table

    def get_export_description(self):
        return [
            ("Nazwa raportu:", "raport slotów - uczelnia"),
            ("Od roku:", self.parent_object.od_roku),
            ("Do roku:", self.parent_object.do_roku),
            ("Maksymalny slot:", self.parent_object.slot),
            (
                "Dziel na jednostki:",
                "tak" if self.parent_object.dziel_na_jednostki_i_wydzialy else "nie",
            ),
            ("Wygenerowano:", self.parent_object.finished_on),
            ("Wersja oprogramowania BPP", VERSION),
        ]

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(**kwargs)
        context["object"] = self.parent_object
        context["export_link"] = urllib.parse.urlencode(
            dict(self.request.GET, **{"_export": "xlsx"}), doseq=True
        )
        context["filter"] = self.get_filterset(self.get_filterset_class())
        return context

    def get_export_filename(self, export_format):
        stamp = timezone.now().strftime("%Y%m%d-%H%M")
        return f"raport_dyscyplin_{self.parent_object.od_roku}-{self.parent_object.do_roku}_{stamp}.{export_format}"

    def get(self, *args, **kw):
        filterset_class = self.get_filterset_class()
        self.filterset = self.get_filterset(filterset_class)

        if (
            not self.filterset.is_bound
            or self.filterset.is_valid()
            or not self.get_strict()
        ):
            self.object_list = self.filterset.qs
        else:
            self.object_list = self.filterset.queryset.none()

        self.table_data = self.object_list

        return super().get(*args, **kw)



# -*- coding: utf-8 -*-


from django.db import models, migrations


def dodaj_dane(apps, schema_editor):
    Jezyk = apps.get_model('bpp', 'Jezyk')

    pl, created = Jezyk.objects.get_or_create(skrot='pol.', nazwa='polski')
    pl.skrot_dla_pbn = 'PL'
    pl.save()
    assert pl.pk == 1

    ang, created = Jezyk.objects.get_or_create(skrot='ang.', nazwa='angielski')
    ang.skrot_dla_pbn = 'EN'
    ang.save()
    assert ang.pk == 2

class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0021_auto_20150825_2301'),
    ]

    operations = [
        migrations.RunPython(dodaj_dane)
    ]



import pytest
from django.urls import reverse
from selenium.webdriver.support.expected_conditions import alert_is_present

from bpp.tests import assertPopupContains, proper_click_element, show_element
from bpp.views.api import const

from django_bpp.selenium_util import wait_for, wait_for_page_load


@pytest.mark.parametrize(
    "tytul,wynik",
    [
        (const.PUBMED_TITLE_NONEXISTENT, const.PUBMED_PO_TYTULE_BRAK),
        (const.PUBMED_TITLE_MULTIPLE, const.PUBMED_PO_TYTULE_WIELE),
        ("", "Aby wykonać zapytanie, potrzebny jest tytuł w polu"),
        ("   ", const.PUBMED_BRAK_PARAMETRU),
    ],
)
@pytest.mark.parametrize(
    "url",
    [
        "wydawnictwo_zwarte",
        "wydawnictwo_ciagle",
    ],
)
def test_change_form_pubmed_brak_takiej_pracy(
    admin_browser, asgi_live_server, url, tytul, wynik
):
    url = reverse(f"admin:bpp_{url}_add")
    with wait_for_page_load(admin_browser):
        admin_browser.visit(asgi_live_server.url + url)
    admin_browser.find_by_id("id_tytul_oryginalny").fill(tytul)
    for elem in admin_browser.find_by_tag("h2")[:3]:
        show_element(admin_browser, elem)  # ._element)
        elem.click()
    if not admin_browser.find_by_id("id_pubmed_id_get"):
        raise Exception("Nie mozna znalexc elementu")

    btn = admin_browser.find_by_id("id_pubmed_id_get")
    proper_click_element(admin_browser, btn)

    wait_for(lambda: alert_is_present()(admin_browser.driver))
    assertPopupContains(admin_browser, wynik)



import pytest
from django.urls import reverse
from selenium.webdriver.support.wait import WebDriverWait

from bpp.models import Autor_Dyscyplina
from bpp.tests import select_select2_autocomplete

from django_bpp.selenium_util import SHORT_WAIT_TIME, wait_for, wait_for_page_load


def scroll_until_handler_clicked_successfully(browser, handler="grp-add-handler"):
    browser.execute_script(
        'document.getElementsByClassName("%s")[0].scrollIntoView();' % handler
    )
    browser.execute_script(
        'document.getElementsByClassName("%s")[0].click();' % handler
    )


@pytest.mark.parametrize("url", ["wydawnictwo_ciagle", "wydawnictwo_zwarte"])
def test_podpowiedzi_dyscyplin_autor_ma_dwie(
    url,
    asgi_live_server,
    admin_browser,
    autor_jan_kowalski,
    dyscyplina1,
    dyscyplina2,
):
    Autor_Dyscyplina.objects.create(
        rok=2018,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )
    url = reverse("admin:bpp_%s_add" % url)
    admin_browser.visit(asgi_live_server.url + url)

    admin_browser.type("rok", "2018")

    scroll_until_handler_clicked_successfully(admin_browser)

    wait_for(lambda: admin_browser.find_by_id("id_autorzy_set-0-autor"))
    select_select2_autocomplete(admin_browser, "id_autorzy_set-0-autor", "KOWALSKI")

    sel = admin_browser.find_by_id("id_autorzy_set-0-dyscyplina_naukowa")
    assert sel.value == "---------"


@pytest.mark.parametrize("url", ["wydawnictwo_ciagle", "wydawnictwo_zwarte"])
def test_podpowiedzi_dyscyplin_autor_ma_jedna_uczelnia_podpowiada(
    url,
    asgi_live_server,
    admin_browser,
    autor_jan_kowalski,
    dyscyplina1,
    dyscyplina2,
    uczelnia,
):
    uczelnia.podpowiadaj_dyscypliny = True
    uczelnia.save()

    Autor_Dyscyplina.objects.create(
        rok=2018, autor=autor_jan_kowalski, dyscyplina_naukowa=dyscyplina1
    )
    url = reverse("admin:bpp_%s_add" % url)

    with wait_for_page_load(admin_browser):
        admin_browser.visit(asgi_live_server.url + url)

    admin_browser.type("rok", "2018")

    scroll_until_handler_clicked_successfully(admin_browser)

    wait_for(lambda: admin_browser.find_by_id("id_autorzy_set-0-autor"))

    select_select2_autocomplete(admin_browser, "id_autorzy_set-0-autor", "KOWALSKI")

    admin_browser.find_by_id("id_autorzy_set-0-dyscyplina_naukowa")

    WebDriverWait(admin_browser.driver, SHORT_WAIT_TIME).until(
        lambda driver: admin_browser.find_by_id(
            "id_autorzy_set-0-dyscyplina_naukowa"
        ).value
        == str(dyscyplina1.pk)
    )


@pytest.mark.parametrize("url", ["wydawnictwo_ciagle", "wydawnictwo_zwarte"])
def test_podpowiedzi_dyscyplin_autor_ma_jedna_uczelnia_nie_podpowiada(
    url,
    asgi_live_server,
    admin_browser,
    autor_jan_kowalski,
    dyscyplina1,
    dyscyplina2,
    uczelnia,
):
    uczelnia.podpowiadaj_dyscypliny = False
    uczelnia.save()

    Autor_Dyscyplina.objects.create(
        rok=2018, autor=autor_jan_kowalski, dyscyplina_naukowa=dyscyplina1
    )
    url = reverse("admin:bpp_%s_add" % url)
    admin_browser.visit(asgi_live_server.url + url)

    admin_browser.type("rok", "2018")

    scroll_until_handler_clicked_successfully(admin_browser)

    wait_for(lambda: admin_browser.find_by_id("id_autorzy_set-0-autor"))
    select_select2_autocomplete(admin_browser, "id_autorzy_set-0-autor", "KOWALSKI")

    sel = admin_browser.find_by_id("id_autorzy_set-0-dyscyplina_naukowa")
    assert sel.value == "---------"



from rest_framework.pagination import PageNumberPagination

from django.utils.functional import cached_property

from bpp.models import Uczelnia


class UkryjStatusyKorektyMixin:
    @cached_property
    def ukryte_statusy(self):
        uczelnia = Uczelnia.objects.get_for_request(self.request)
        if uczelnia:
            ukryte_statusy = uczelnia.ukryte_statusy("api")
            return ukryte_statusy

    def get_queryset(self):
        queryset = super().get_queryset()

        if self.ukryte_statusy:
            queryset = queryset.exclude(status_korekty_id__in=self.ukryte_statusy)

        return queryset


class StreszczeniaPagination(PageNumberPagination):
    page_size = 1
    page_size_query_param = "page_size"
    max_page_size = 5



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-08 15:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('import_dyscyplin', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='import_dyscyplin',
            name='web_page_uid',
            field=models.CharField(blank=True, max_length=36, null=True),
        ),
    ]



# -*- encoding: utf-8 -*-
import lxml.html
from django.template import Library

from django.utils.safestring import mark_safe

register = Library()


def strip_at_end(ciag, znaki=",."):
    ciag = ciag.strip()
    while ciag:
        if ciag[-1] in znaki:
            ciag = ciag[:-1]
            continue
        break
    return ciag


def strip_at_beginning(ciag, znaki=",."):
    ciag = ciag.strip()
    while ciag:
        if ciag[0] in znaki:
            ciag = ciag[1:]
            continue
        break
    return ciag


def znak_na_koncu(ciag, znak):
    """Wymusza, aby na końcu ciągu znaków był konkretny znak, czyli przecinek
    albo kropka. Wycina wszelkie kropki i przecinki z końca ciągu i stripuje go,
    zwracając
    """
    if ciag is None:
        return

    ciag = strip_at_end(ciag)
    if ciag:
        return ciag + znak
    return ciag


register.filter(znak_na_koncu)


def znak_na_poczatku(ciag, znak):
    """Wymusza, aby na PRZED ciągiem znaków był konkretny znak ORAZ spacja,
    czyli - przykładowo - przecinek albo kropka, jeżeli ciąg jest nie-pusty;
    do tego wycina wszelkie kropki i przecinki z końca i z początku ciągu
    oraz stripuje go.

    Tag używany do uzyskiwania opisu bibliograficznego.
    """
    if ciag is None:
        return ""

    ciag = strip_at_beginning(strip_at_end(ciag))
    if ciag:
        return znak + " " + ciag
    return ciag


register.filter(znak_na_poczatku)


def ladne_numery_prac(arr):
    """Wyświetla ładne numery prac, tzn. tablicę [1, 2, 5, 6, 7, 8, 12]
    przerobi na 1-2, 5-8, 12

    Filtr wykorzystywany do wyświetlania numerków prac w Kronice Uczelni
    """

    # To może być set(), a set() jest nieposortowany
    nu = sorted(arr)

    if not nu:
        return ""

    buf = str(nu[0])
    last_elem = nu[0]
    cont = False

    for elem in nu[1:]:
        if elem == last_elem + 1:
            last_elem = elem
            cont = True
            continue

        if cont:
            buf += "-" + str(last_elem) + ", " + str(elem)
        else:
            buf += ", " + str(elem)

        last_elem = elem
        cont = False

    if cont:
        buf += "-" + str(last_elem)

    return buf


register.filter(ladne_numery_prac)


@register.simple_tag
def opis_bibliograficzny_cache(pk):
    from bpp.models.cache import Rekord

    try:
        return mark_safe(Rekord.objects.get(pk=pk).opis_bibliograficzny_cache)
    except Rekord.DoesNotExist:
        pass

    return "(brak danych)"


CLOSE_TAGS_OPENING = "<html><body><foo>"
CLOSE_TAGS_CLOSING = "</foo></body></html>"


@register.filter(name="close_tags")
def close_tags(s):
    if s is None or not s:
        return s
    s = f"{CLOSE_TAGS_OPENING}{s}{CLOSE_TAGS_CLOSING}"
    s = lxml.html.fromstring(s)
    s = lxml.etree.tostring(s, encoding="unicode")
    s = s[len(CLOSE_TAGS_OPENING) : -len(CLOSE_TAGS_CLOSING)]
    return s



# Generated by Django 3.0.14 on 2021-10-27 21:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ewaluacja2021", "0003_auto_20211027_2320"),
    ]

    operations = [
        migrations.AddField(
            model_name="importmaksymalnychslotow",
            name="ostatnia_zmiana",
            field=models.DateTimeField(auto_now=True),
        ),
    ]



from queryset_sequence import QuerySetSequence

from pbn_api.admin import MaMNISWIDFilter
from pbn_api.admin.base import BaseMongoDBAdmin
from pbn_api.admin.filters import OdpowiednikWydawcyWBPPFilter
from pbn_api.models import Publisher

from django.contrib import admin
from django.contrib.postgres.search import TrigramSimilarity

from bpp.const import PBN_UID_LEN


@admin.register(Publisher)
class PublisherAdmin(BaseMongoDBAdmin):
    list_display = ["publisherName", "mniswId", "mongoId", "rekord_w_bpp"]
    search_fields = ["mongoId", "publisherName", "mniswId"]
    list_filter = [
        OdpowiednikWydawcyWBPPFilter,
        MaMNISWIDFilter,
    ] + BaseMongoDBAdmin.list_filter

    MIN_TRIGRAM_MATCH = 0.1

    def get_search_results(self, request, queryset, search_term):
        if not search_term or len(search_term) == PBN_UID_LEN:
            return super().get_search_results(request, queryset, search_term)

        bazowe_zapytanie = (
            queryset.annotate(
                similarity=TrigramSimilarity("publisherName", search_term)
            )
            .filter(similarity__gte=self.MIN_TRIGRAM_MATCH)
            .order_by("-similarity")
        )

        z_identyfikatorami = bazowe_zapytanie.exclude(mniswId=None)[:10]
        bez_identyfikatorow = bazowe_zapytanie.filter(mniswId=None)[:10]

        return QuerySetSequence(z_identyfikatorami, bez_identyfikatorow), False



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-06-16 20:01


import autoslug.fields
import django.contrib.auth.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0073_auto_20170616_1709'),
    ]

    operations = [
        migrations.AlterField(
            model_name='autor',
            name='slug',
            field=autoslug.fields.AutoSlugField(editable=False, max_length=1024, populate_from=b'get_full_name', unique=True),
        ),
        migrations.AlterField(
            model_name='bppuser',
            name='username',
            field=models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.ASCIIUsernameValidator()], verbose_name='username'),
        ),
        migrations.AlterField(
            model_name='jednostka',
            name='slug',
            field=autoslug.fields.AutoSlugField(editable=False, populate_from=b'nazwa', unique=True),
        ),
        migrations.AlterField(
            model_name='uczelnia',
            name='slug',
            field=autoslug.fields.AutoSlugField(editable=False, populate_from=b'skrot', unique=True),
        ),
        migrations.AlterField(
            model_name='wydzial',
            name='slug',
            field=autoslug.fields.AutoSlugField(editable=False, max_length=512, populate_from=b'nazwa', unique=True),
        ),
        migrations.AlterField(
            model_name='zrodlo',
            name='slug',
            field=autoslug.fields.AutoSlugField(editable=False, populate_from=b'nazwa', unique=True),
        ),
    ]



# Generated by Django 3.0.14 on 2021-09-15 19:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0292_przypinanie_dyscyplin"),
    ]

    operations = [
        migrations.AddField(
            model_name="uczelnia",
            name="pbn_api_kasuj_przed_wysylka",
            field=models.BooleanField(
                default=False,
                verbose_name="Kasuj oświadczenia rekordu przed wysłaniem do PBN",
            ),
        ),
        migrations.AddField(
            model_name="uczelnia",
            name="pbn_api_nie_wysylaj_prac_bez_pk",
            field=models.BooleanField(
                default=False, verbose_name="Nie wysyłaj do PBN prac z PK=0"
            ),
        ),
    ]



# Generated by Django 3.2.15 on 2022-09-10 19:14

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("zglos_publikacje", "0012_auto_20220910_1654"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="obslugujacy_zgloszenia_wydzialow",
            managers=[],
        ),
        migrations.AddField(
            model_name="zgloszenie_publikacji",
            name="utworzyl",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
                verbose_name="Utworzył",
            ),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0017_typy_pbn'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='bppuser',
            options={'verbose_name': 'u\u017cytkownik', 'verbose_name_plural': 'u\u017cytkownicy'},
        ),
    ]



# -*- encoding: utf-8 -*-

from django.core.management import BaseCommand
from django.db import transaction

from bpp.models.cache import Rekord
from bpp.models import cache


FIXME = [
    ('<alpha>', 'α'),
    ('<alfa>', 'α'),
    ('<afa>', 'α'),

    ('<beta>', 'β'),
    ('<gamma>', 'γ'),
    ('<delta>', 'δ'),
    ('<d>', 'δ'),
    ('<epsilon>', 'ε'),
    ('<pi>', 'π'),

    ('<mi>', 'μ'),
    ('<omega>', 'ω'),

    ('<zeta>', 'ζ'),
    ('<tau>', 'τ'),
    ('<kappa>', 'κ'),

    ('<sub)', '<sub>'),

    ('<dn>', '<sub>'),
    ('</dn>', '</sub>'),

    ('<up>', '<sup>'),
    ('</up>', '</sup>'),

    ('<Heraclem sibiricum', '<i>Heraclem sibiricum</i>'),
    ('<foramen jugulare>', '<i>foramen jugulare</i>'),

]

class Command(BaseCommand):
    help = 'Naprawia tytuły prac po imporcie'

    @transaction.atomic
    def handle(self, *args, **options):
        cache.enable()

        for string, replacement in FIXME:
            for rekord in Rekord.objects.filter(tytul_oryginalny__contains=string):
                pre = (rekord.tytul_oryginalny, rekord.tytul)
                for pole in ['tytul', 'tytul_oryginalny']:
                    setattr(rekord, pole, getattr(rekord, pole).replace(string, replacement))
                post = (rekord.tytul_oryginalny, rekord.tytul)

                if pre != post:
                    # print pre, post
                    pass

                rekord.save()

        for elem in Rekord.objects.raw("""select * from bpp_rekord_mat where
tytul_oryginalny like '%%<%%' and
lower(tytul_oryginalny) not like '%%<b>%%'  and
lower(tytul_oryginalny) not like '%%<i>%%'  and
lower(tytul_oryginalny) not like '%%<sub>%%'  and
lower(tytul_oryginalny) not like '%%<sup>%%'  and
lower(tytul_oryginalny) not like '%%<beta>%%' and
lower(tytul_oryginalny) not like '%%<alfa>%%' and
lower(tytul_oryginalny) not like '%%<pi>%%'  and
lower(tytul_oryginalny) not like '%%<gamma>%%' and
lower(tytul_oryginalny) not like '%%<up>%%' and
lower(tytul_oryginalny) not like '%%<dn>%%' and
lower(tytul_oryginalny) not like '%%<mi>%%' and
lower(tytul_oryginalny) not like '%%<i/>%%' and
lower(tytul_oryginalny) not like '%%<alfa>%%' and
lower(tytul_oryginalny) not like '%%<afa>%%' and
lower(tytul_oryginalny) not like '%%<delta>%%'  and
lower(tytul_oryginalny) not like '%%<zeta>%%'  and
lower(tytul_oryginalny) not like '%%<tau>%%'  and
lower(tytul_oryginalny) not like '%%<kappa>%%'  and
lower(tytul_oryginalny) not like '%%<sub)%%'  and
lower(tytul_oryginalny) not like '%%<alpha>%%';
"""):
            print("%r" % elem.tytul_oryginalny)


from django.db import transaction
from django.http import JsonResponse
from django.http.response import HttpResponseNotFound
from django.views.generic import View

from bpp.models import Autor, Autor_Dyscyplina, Uczelnia, Zrodlo
from bpp.models.abstract import POLA_PUNKTACJI
from bpp.models.praca_habilitacyjna import Praca_Habilitacyjna
from bpp.models.zrodlo import Punktacja_Zrodla


class RokHabilitacjiView(View):
    def post(self, request, *args, **kw):
        try:
            autor = Autor.objects.get(pk=int(request.POST.get("autor_pk")))
        except Autor.DoesNotExist:
            return HttpResponseNotFound("Autor")

        try:
            habilitacja = autor.praca_habilitacyjna
        except Praca_Habilitacyjna.DoesNotExist:
            return HttpResponseNotFound("Habilitacja")

        return JsonResponse({"rok": habilitacja.rok})


class PunktacjaZrodlaView(View):
    def post(self, request, zrodlo_id, rok, *args, **kw):

        try:
            z = Zrodlo.objects.get(pk=zrodlo_id)
        except Zrodlo.DoesNotExist:
            return HttpResponseNotFound("Zrodlo")

        try:
            pz = Punktacja_Zrodla.objects.get(zrodlo=z, rok=rok)
        except Punktacja_Zrodla.DoesNotExist:
            return HttpResponseNotFound("Rok")

        # d = dict([(pole, str(getattr(pz, pole))) for pole in POLA_PUNKTACJI])
        d = {pole: getattr(pz, pole) for pole in POLA_PUNKTACJI}
        return JsonResponse(d)


class UploadPunktacjaZrodlaView(View):
    def ok(self):
        return JsonResponse({"result": "ok"})

    @transaction.atomic
    def post(self, request, zrodlo_id, rok, *args, **kw):
        try:
            z = Zrodlo.objects.get(pk=zrodlo_id)
        except Zrodlo.DoesNotExist:
            return HttpResponseNotFound("Zrodlo")

        kw_punktacji = {}
        for element in list(request.POST.keys()):
            if element in POLA_PUNKTACJI:
                if request.POST.get(element) != "":
                    kw_punktacji[element] = request.POST.get(element) or "0.0"

        try:
            pz = Punktacja_Zrodla.objects.get(zrodlo=z, rok=rok)
        except Punktacja_Zrodla.DoesNotExist:
            Punktacja_Zrodla.objects.create(zrodlo=z, rok=rok, **kw_punktacji)
            return self.ok()

        if request.POST.get("overwrite") == "1":
            for key, value in list(kw_punktacji.items()):
                setattr(pz, key, value or "0.0")
            pz.save()
            return self.ok()
        return JsonResponse(dict(result="exists"))


class OstatniaJednostkaIDyscyplinaView(View):
    """Zwraca jako JSON ostatnią jednostkę danego autora oraz ewentualnie jego
    dyscyplinę naukową, w sytuacji gdy jest ona jedna i określona na dany rok.
    """

    def post(self, request, *args, **kw):
        try:
            a = Autor.objects.get(pk=int(request.POST.get("autor_id", None)))
        except (Autor.DoesNotExist, TypeError, ValueError):
            return JsonResponse({"status": "error", "reason": "autor nie istnieje"})

        uczelnia = Uczelnia.objects.get_for_request(request)

        ret = {}

        jed = a.aktualna_jednostka
        if jed is None:
            # Brak aktualnej jednostki, spróbuj podpowiedzieć obcą jednostkę:
            if uczelnia is None:
                ret.update(dict(jednostka_id=None, nazwa=None, status="ok"))
            else:
                # uczelnia is not None
                if uczelnia.obca_jednostka_id is not None:
                    ret.update(
                        dict(
                            jednostka_id=uczelnia.obca_jednostka_id,
                            nazwa=uczelnia.obca_jednostka.nzawa,
                            status="ok",
                        )
                    )
        else:
            ret.update(dict(jednostka_id=jed.pk, nazwa=jed.nazwa, status="ok"))

        try:
            rok = int(request.POST.get("rok"))
        except (TypeError, ValueError):
            rok = None

        if uczelnia is not None and uczelnia.podpowiadaj_dyscypliny and rok:
            ad = Autor_Dyscyplina.objects.filter(autor=a, rok=rok)

            if ad.exists():
                # Jest wpis Autor_Dyscyplina dla tego autora i roku.

                ad = ad.first()

                if not (
                    ad.dyscyplina_naukowa_id is not None
                    and ad.subdyscyplina_naukowa_id is not None
                ):
                    # W sytuacji gdy określona jest tylko jedna z nich:

                    d = ad.dyscyplina_naukowa or ad.subdyscyplina_naukowa
                    ret["dyscyplina_nazwa"] = d.nazwa
                    ret["dyscyplina_id"] = d.pk
                    ret["status"] = "ok"

        return JsonResponse(ret)



from django.db import models
from django.db.models import CASCADE, DO_NOTHING, JSONField

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType


class Bib(models.Model):
    idt = models.IntegerField(primary_key=True)
    tytul_or = models.TextField(blank=True, null=True)
    title = models.TextField(blank=True, null=True)
    zrodlo = models.TextField(blank=True, null=True)
    szczegoly = models.TextField(blank=True, null=True)
    uwagi = models.TextField(blank=True, null=True)
    charakter = models.TextField(blank=True, null=True)
    impact = models.TextField(blank=True, null=True)
    redakcja = models.TextField(blank=True, null=True)
    status = models.TextField(blank=True, null=True)
    rok = models.TextField(blank=True, null=True)
    sort = models.TextField(blank=True, null=True)
    sort2 = models.TextField(blank=True, null=True)
    export = models.TextField(blank=True, null=True)
    import_field = models.TextField(
        db_column="import", blank=True, null=True
    )  # Field renamed because it was a Python reserved word.
    naz_imie = models.TextField(blank=True, null=True)
    redaktor = models.TextField(blank=True, null=True)
    redaktor0 = models.TextField(blank=True, null=True)
    tytul_or_s = models.TextField(blank=True, null=True)
    title_s = models.TextField(blank=True, null=True)
    zrodlo_s = models.TextField(blank=True, null=True)
    szczegol_s = models.TextField(blank=True, null=True)
    mem_fi_ext = models.TextField(blank=True, null=True)
    dat_akt = models.TextField(blank=True, null=True)
    kbn = models.TextField(blank=True, null=True)
    kbr = models.TextField(blank=True, null=True)
    afiliowana = models.TextField(blank=True, null=True)
    recenzowan = models.TextField(blank=True, null=True)
    jezyk = models.TextField(blank=True, null=True)
    jezyk2 = models.TextField(blank=True, null=True)
    punkty_kbn = models.TextField(db_column="pk", blank=True, null=True)
    x_skrot = models.TextField(blank=True, null=True)
    wspx = models.TextField(blank=True, null=True)
    x2_skrot = models.TextField(blank=True, null=True)
    wspx2 = models.TextField(blank=True, null=True)
    y_skrot = models.TextField(blank=True, null=True)
    wspy = models.TextField(blank=True, null=True)
    wspq = models.TextField(blank=True, null=True)
    ic = models.TextField(blank=True, null=True)
    rok_inv = models.TextField(blank=True, null=True)
    link = models.TextField(blank=True, null=True)
    lf = models.TextField(blank=True, null=True)
    rok_punkt = models.TextField(blank=True, null=True)
    form = models.TextField(blank=True, null=True)
    k_z = models.TextField(blank=True, null=True)
    uwagi2 = models.TextField(blank=True, null=True)
    dat_utw = models.TextField(blank=True, null=True)
    pun_wl = models.TextField(blank=True, null=True)
    study_gr = models.TextField(blank=True, null=True)
    sort_fixed = models.TextField(blank=True, null=True)
    zaznacz_field = models.TextField(
        db_column="zaznacz_", blank=True, null=True
    )  # Field renamed because it ended with '_'.
    idt2 = models.ForeignKey("self", CASCADE, db_column="idt2", blank=True, null=True)
    pun_max = models.TextField(blank=True, null=True)
    pun_erih = models.TextField(blank=True, null=True)
    kwartyl = models.TextField(blank=True, null=True)
    issn = models.TextField(blank=True, null=True)
    eissn = models.TextField(blank=True, null=True)
    wok_id = models.TextField(blank=True, null=True)
    sco_id = models.TextField(blank=True, null=True)
    mnsw_fixed = models.TextField(blank=True, null=True)
    liczba_aut = models.TextField(blank=True, null=True)
    pro_p_wydz = models.TextField(blank=True, null=True)
    snip = models.TextField(blank=True, null=True)
    sjr = models.TextField(blank=True, null=True)
    cites = models.TextField(blank=True, null=True)
    if5 = models.TextField(blank=True, null=True)
    lis_numer = models.TextField(blank=True, null=True)

    object_id = models.PositiveIntegerField(null=True, blank=True)
    content_type = models.ForeignKey(ContentType, DO_NOTHING, null=True, blank=True)
    object = GenericForeignKey()

    analyzed = models.BooleanField(default=False)

    class Meta:
        db_table = "import_dbf_bib"
        verbose_name = "zaimportowany rekord bibliografi"
        verbose_name_plural = "zaimportowane rekordy bibliografi"

    def __str__(self):
        return self.tytul_or_s


class Bib_Desc(models.Model):
    idt = models.ForeignKey(Bib, CASCADE)
    elem_id = models.PositiveSmallIntegerField(db_index=True)
    value = JSONField()
    source = models.CharField(max_length=10)

    class Meta:
        ordering = ("idt", "source")


class Aut(models.Model):
    idt_aut = models.TextField(primary_key=True)
    imiona = models.TextField(blank=True, null=True)
    nazwisko = models.TextField(blank=True, null=True)
    # 'ref' to odnośnik do sposobu zapisania danego autora w publikacji (myśl: b_a)
    ref = models.ForeignKey(
        "import_dbf.Aut",
        blank=True,
        null=True,
        db_column="ref",
        on_delete=DO_NOTHING,
        related_name="aut_ref",
    )
    idt_jed = models.ForeignKey(
        "import_dbf.Jed",
        blank=True,
        null=True,
        db_column="idt_jed",
        on_delete=DO_NOTHING,
    )
    kad_nr = models.TextField(blank=True, null=True)
    tel = models.TextField(blank=True, null=True)
    email = models.TextField(blank=True, null=True)
    www = models.TextField(blank=True, null=True)
    imiona_bz = models.TextField(blank=True, null=True)
    nazwisk_bz = models.TextField(blank=True, null=True)
    tytul = models.TextField(blank=True, null=True)
    stanowisko = models.TextField(blank=True, null=True)
    prac_od = models.TextField(blank=True, null=True)
    dat_zwol = models.TextField(blank=True, null=True)
    fg = models.TextField(blank=True, null=True)
    dop = models.TextField(blank=True, null=True)
    nr_ewid = models.TextField(blank=True, null=True)
    kad_s_jed = models.TextField(blank=True, null=True)
    pbn_id = models.TextField(blank=True, null=True)
    res_id = models.TextField(blank=True, null=True)
    scop_id = models.TextField(blank=True, null=True)
    orcid_id = models.TextField(blank=True, null=True)
    # exp_id to autor "nadrzędny"(?)
    exp_id = models.ForeignKey(
        "import_dbf.Aut",
        blank=True,
        null=True,
        db_column="exp_id",
        on_delete=DO_NOTHING,
        related_name="aut_exp",
    )
    polon_id = models.TextField(blank=True, null=True)
    usos_id = models.TextField(blank=True, null=True)
    udf_id = models.TextField(blank=True, null=True)
    control = models.TextField(blank=True, null=True)
    uwagi = models.TextField(blank=True, null=True)
    graf = models.TextField(blank=True, null=True)

    bpp_autor = models.ForeignKey(
        "bpp.Autor", null=True, blank=True, on_delete=DO_NOTHING
    )

    class Meta:
        managed = False
        db_table = "import_dbf_aut"
        verbose_name = "zaimportowany autor"
        verbose_name_plural = "zaimportowani autorzy"
        ordering = ("nazwisko", "imiona")

    def __str__(self):
        ret = f"{self.nazwisko} {self.imiona}"
        if self.tytul:
            ret += f", {self.tytul}"
        return ret

    def get_bpp(self):
        return self.bpp_autor


class Jed(models.Model):
    idt_jed = models.TextField(primary_key=True)
    nr = models.TextField(blank=True, null=True)
    skrot = models.TextField(blank=True, null=True)
    nazwa = models.TextField(blank=True, null=True)
    wyd_skrot = models.TextField(blank=True, null=True)
    sort = models.TextField(blank=True, null=True)
    to_print = models.TextField(blank=True, null=True)
    to_print2 = models.TextField(blank=True, null=True)
    to_print3 = models.TextField(blank=True, null=True)
    to_print4 = models.TextField(blank=True, null=True)
    to_print5 = models.TextField(blank=True, null=True)
    email = models.TextField(blank=True, null=True)
    www = models.TextField(blank=True, null=True)
    id_u = models.TextField(blank=True, null=True)

    bpp_jednostka = models.ForeignKey(
        "bpp.Jednostka", DO_NOTHING, blank=True, null=True
    )

    def get_bpp(self):
        return self.bpp_jednostka

    class Meta:
        managed = False
        db_table = "import_dbf_jed"

    def __str__(self):
        return self.nazwa


class B_A(models.Model):
    id = models.IntegerField(primary_key=True)
    idt = models.ForeignKey("import_dbf.Bib", db_column="idt", on_delete=DO_NOTHING)
    lp = models.TextField(blank=True, null=True)
    idt_aut = models.ForeignKey(
        "import_dbf.Aut", db_column="idt_aut", on_delete=DO_NOTHING
    )
    idt_jed = models.ForeignKey(
        "import_dbf.Jed", db_column="idt_jed", on_delete=DO_NOTHING
    )
    wspz = models.TextField(blank=True, null=True)
    pkt_dod = models.TextField(blank=True, null=True)
    wspz2 = models.TextField(blank=True, null=True)
    pkt2_dod = models.TextField(blank=True, null=True)
    afiliacja = models.TextField(blank=True, null=True)
    odp = models.TextField(blank=True, null=True)
    study_ga = models.TextField(blank=True, null=True)
    tytul = models.TextField(blank=True, null=True)
    stanowisko = models.TextField(blank=True, null=True)
    uwagi = models.TextField(blank=True, null=True)

    object_id = models.PositiveIntegerField(null=True, blank=True)
    content_type = models.ForeignKey(ContentType, DO_NOTHING, null=True, blank=True)
    object = GenericForeignKey()

    def __str__(self):
        return f"Przypisanie autora {self.idt_aut} w jednostce {self.idt_jed} do rekordu {self.idt}"

    class Meta:
        managed = False
        ordering = ("idt__tytul_or_s", "lp")
        db_table = "import_dbf_b_a"


class PozManager(models.Manager):
    def get_for_model(self, idt, rec_type):
        ret = ""
        for elem in self.filter(idt=idt, kod_opisu=rec_type):
            ret += elem.tresc
        return ret


class Poz(models.Model):
    id = models.IntegerField(primary_key=True)
    idt = models.ForeignKey("import_dbf.Bib", db_column="idt", on_delete=DO_NOTHING)
    kod_opisu = models.TextField(blank=True, null=True)
    lp = models.PositiveSmallIntegerField()
    tresc = models.TextField(blank=True, null=True)

    objects = PozManager()

    class Meta:
        managed = False
        ordering = ("idt", "kod_opisu", "lp")
        db_table = "import_dbf_poz"
        verbose_name = "zaimportowany opis rekordu"
        verbose_name_plural = "zaimportowane opisy rekordow"


class B_U(models.Model):
    idt = models.ForeignKey("import_dbf.Bib", DO_NOTHING, db_column="idt")
    idt_usi = models.ForeignKey("import_dbf.Usi", DO_NOTHING, db_column="idt_usi")
    comm = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        verbose_name_plural = "zaimportowane dane OA rekordow"
        verbose_name = "zaimportowane dane OA rekordu"
        db_table = "import_dbf_b_u"
        ordering = (
            "idt",
            "comm",
        )


class Usi(models.Model):
    idt_usi = models.IntegerField(primary_key=True)
    usm_f = models.TextField(blank=True, null=True)
    usm_sf = models.TextField(blank=True, null=True)
    skrot = models.TextField(blank=True, null=True)
    nazwa = models.TextField(blank=True, null=True)
    bpp_id = models.ForeignKey("bpp.Zrodlo", DO_NOTHING, db_column="bpp_id", null=True)
    bpp_wydawca_id = models.ForeignKey(
        "bpp.Wydawca", DO_NOTHING, db_column="bpp_wydawca_id", null=True
    )
    bpp_seria_wydawnicza_id = models.ForeignKey(
        "bpp.Seria_Wydawnicza",
        DO_NOTHING,
        db_column="bpp_seria_wydawnicza_id",
        null=True,
    )

    class Meta:
        managed = False
        verbose_name_plural = "zaimportowane źródła"
        verbose_name = "zaimportowane źródło"
        db_table = "import_dbf_usi"

    def __str__(self):
        return self.nazwa


class Ses(models.Model):
    redaktor = models.TextField(blank=True, null=True)
    file = models.TextField(blank=True, null=True)
    login_t = models.TextField(blank=True, null=True)
    logout_t = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_ses"


class Wx2(models.Model):
    idt_wsx = models.TextField(primary_key=True)
    skrot = models.TextField(blank=True, null=True)
    nazwa = models.TextField(blank=True, null=True)
    wsp = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_wx2"


class Ixn(models.Model):
    idt_pbn = models.TextField(blank=True, primary_key=True)
    pbn = models.TextField(blank=True, null=True)

    def __str__(self):
        return self.pbn

    class Meta:
        managed = False
        verbose_name = "zaimportowany identyfikator PBN"
        verbose_name_plural = "zaimportowane identyfikatory PBN"
        db_table = "import_dbf_ixn"


class B_B(models.Model):
    id = models.IntegerField(primary_key=True, db_column="pk")
    idt = models.ForeignKey("import_dbf.Bib", on_delete=DO_NOTHING, db_column="idt")
    lp = models.TextField(blank=True, null=True)
    idt_bazy = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_b_b"
        ordering = ("idt__tytul_or_s", "lp")


class B_N(models.Model):
    id = models.IntegerField(primary_key=True, db_column="pk")
    idt = models.ForeignKey("import_dbf.bib", on_delete=DO_NOTHING, db_column="idt")
    lp = models.TextField(blank=True, null=True)
    idt_pbn = models.ForeignKey(
        "import_dbf.Ixn", db_column="idt_pbn", on_delete=DO_NOTHING
    )

    class Meta:
        managed = False
        db_table = "import_dbf_b_n"


class Dys(models.Model):
    orcid_id = models.TextField(primary_key=True)
    a_n = models.TextField(blank=True, null=True)
    a_w_etatu = models.TextField(blank=True, null=True)
    a_dysc_1 = models.ForeignKey(
        "import_dbf.Ldy", on_delete=DO_NOTHING, db_column="a_dysc_1", related_name="+"
    )
    a_dysc_2 = models.ForeignKey(
        "import_dbf.Ldy", on_delete=DO_NOTHING, db_column="a_dysc_2", related_name="+"
    )
    a_dysc_1_e = models.TextField(blank=True, null=True)
    a_dysc_2_e = models.TextField(blank=True, null=True)

    b_n = models.TextField(blank=True, null=True)
    b_w_etatu = models.TextField(blank=True, null=True)
    b_dysc_1 = models.ForeignKey(
        "import_dbf.Ldy", on_delete=DO_NOTHING, db_column="b_dysc_1", related_name="+"
    )
    b_dysc_2 = models.ForeignKey(
        "import_dbf.Ldy", on_delete=DO_NOTHING, db_column="b_dysc_2", related_name="+"
    )
    b_dysc_1_e = models.TextField(blank=True, null=True)
    b_dysc_2_e = models.TextField(blank=True, null=True)

    c_n = models.TextField(blank=True, null=True)
    c_w_etatu = models.TextField(blank=True, null=True)
    c_dysc_1 = models.ForeignKey(
        "import_dbf.Ldy", on_delete=DO_NOTHING, db_column="c_dysc_1", related_name="+"
    )
    c_dysc_2 = models.ForeignKey(
        "import_dbf.Ldy", on_delete=DO_NOTHING, db_column="c_dysc_2", related_name="+"
    )
    c_dysc_1_e = models.TextField(blank=True, null=True)
    c_dysc_2_e = models.TextField(blank=True, null=True)

    d_n = models.TextField(blank=True, null=True)
    d_w_etatu = models.TextField(blank=True, null=True)
    d_dysc_1 = models.ForeignKey(
        "import_dbf.Ldy", on_delete=DO_NOTHING, db_column="d_dysc_1", related_name="+"
    )
    d_dysc_2 = models.ForeignKey(
        "import_dbf.Ldy", on_delete=DO_NOTHING, db_column="d_dysc_2", related_name="+"
    )
    d_dysc_1_e = models.TextField(blank=True, null=True)
    d_dysc_2_e = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        verbose_name = "zaimportowana dyscyplina pracownika"
        verbose_name_plural = "zaimportowane dyscypliny pracowników"
        db_table = "import_dbf_dys"


class Ixe(models.Model):
    idt_eng = models.TextField(primary_key=True)
    haslo = models.TextField(blank=True, null=True)

    def __str__(self):
        return self.haslo

    class Meta:
        managed = False
        verbose_name = "zaimportowane hasło naukowe"
        verbose_name_plural = "zaimportowane hasła naukowe"
        db_table = "import_dbf_ixe"


class Jer(models.Model):
    id = models.IntegerField(primary_key=True, db_column="pk")
    nr = models.TextField()
    od_roku = models.TextField(blank=True, null=True)
    skrot = models.TextField(blank=True, null=True)
    nazwa = models.TextField(blank=True, null=True)
    wyd_skrot = models.TextField(blank=True, null=True)
    id_u = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_jer"


class Kad(models.Model):
    nr = models.TextField(primary_key=True)
    na = models.TextField(blank=True, null=True)
    im1 = models.TextField(blank=True, null=True)
    im2 = models.TextField(blank=True, null=True)
    s_jed = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_kad"


class Loc(models.Model):
    ident = models.TextField(primary_key=True)
    ext = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_loc"


class Pbc(models.Model):
    idt = models.TextField(primary_key=True)
    wyd_skrot = models.TextField(blank=True, null=True)
    date = models.TextField(blank=True, null=True)
    category = models.TextField(blank=True, null=True)
    details = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_pbc"


class Pub(models.Model):
    idt_pub = models.TextField(primary_key=True)
    skrot = models.TextField(blank=True, null=True)
    nazwa = models.TextField(blank=True, null=True)
    to_print = models.TextField(blank=True, null=True)
    to_print2 = models.TextField(blank=True, null=True)
    to_print3 = models.TextField(blank=True, null=True)
    to_print4 = models.TextField(blank=True, null=True)
    to_print5 = models.TextField(blank=True, null=True)
    bpp_id = models.ForeignKey(
        "bpp.Charakter_Formalny", DO_NOTHING, db_column="bpp_id", null=True
    )

    class Meta:
        managed = False
        verbose_name = "zaimportowany charakter publikacji"
        verbose_name_plural = "zaimportowane charaktery publikacji"
        db_table = "import_dbf_pub"


class Sci(models.Model):
    idt_sci = models.TextField(primary_key=True)
    au = models.TextField(blank=True, null=True)
    ti = models.TextField(blank=True, null=True)
    src = models.TextField(blank=True, null=True)
    ye = models.TextField(blank=True, null=True)
    cont = models.TextField(blank=True, null=True)
    ut = models.TextField(blank=True, null=True)
    field_ignore_me = models.TextField(
        db_column="_ignore_me", blank=True, null=True
    )  # Field renamed because it started with '_'.

    class Meta:
        managed = False
        db_table = "import_dbf_sci"


class Sys(models.Model):
    ver = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_sys"
        verbose_name = "zaimportowana wersja bazy"
        verbose_name_plural = "zaimportowane wersje bazy"

    def __str__(self):
        return self.ver


class Wsx(models.Model):
    idt_wsx = models.TextField(primary_key=True)
    skrot = models.TextField(blank=True, null=True)
    nazwa = models.TextField(blank=True, null=True)
    wsp = models.TextField(blank=True, null=True)
    field_ignore_me = models.TextField(
        db_column="_ignore_me", blank=True, null=True
    )  # Field renamed because it started with '_'.

    class Meta:
        managed = False
        db_table = "import_dbf_wsx"


class Wyd(models.Model):
    idt_wyd = models.TextField(primary_key=True)
    skrot = models.TextField(blank=True, null=True)
    nazwa = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_wyd"
        verbose_name = "zaimportowany wydział"
        verbose_name_plural = "zaimportowane wydzialy"

    def __str__(self):
        return self.nazwa


class Ldy(models.Model):
    id = models.TextField(primary_key=True)
    dziedzina = models.TextField(blank=True, null=True)
    dyscyplina = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_ldy"
        verbose_name = "zaimportowana dziedzina"
        verbose_name_plural = "zaimportowane dziedziny"

    def __str__(self):
        return f"{self.dziedzina} / {self.dyscyplina}"


class B_E(models.Model):
    idt = models.IntegerField()
    lp = models.TextField(blank=True, null=True)
    idt_eng = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_b_e"


class B_P(models.Model):
    idt = (
        models.IntegerField()
    )  # ForeignKey("import_dbf.Bib", db_column='idt', on_delete=DO_NOTHING)
    lp = models.TextField(blank=True, null=True)
    idt_pol = models.ForeignKey(
        "import_dbf.Ixp", db_column="idt_pol", on_delete=DO_NOTHING
    )

    class Meta:
        managed = False
        db_table = "import_dbf_b_p"

    def __str__(self):
        return "powiązanie B_P dla %s" % self.idt


class Ixp(models.Model):
    idt_pol = models.TextField(primary_key=True)
    haslo = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_ixp"

    def __str__(self):
        return self.haslo


class Jez(models.Model):
    skrot = models.TextField(primary_key=True)
    nazwa = models.TextField(blank=True, null=True)
    bpp_id = models.ForeignKey("bpp.Jezyk", DO_NOTHING, db_column="bpp_id", null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_jez"
        verbose_name = "zaimportowany język"
        verbose_name_plural = "zaimportowane języki"

    def __str__(self):
        return self.nazwa


class Kbn(models.Model):
    idt_kbn = models.TextField(primary_key=True)
    skrot = models.TextField(blank=True, null=True)
    nazwa = models.TextField(blank=True, null=True)
    to_print = models.TextField(blank=True, null=True)
    to_print2 = models.TextField(blank=True, null=True)
    to_print3 = models.TextField(blank=True, null=True)
    to_print4 = models.TextField(blank=True, null=True)
    to_print5 = models.TextField(blank=True, null=True)

    bpp_id = models.ForeignKey("bpp.Typ_KBN", DO_NOTHING, db_column="bpp_id", null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_kbn"
        verbose_name = "zaimportowany typ KBN"
        verbose_name_plural = "zaimportowane typy KBN"

    def __str__(self):
        return self.nazwa


class Pba(models.Model):
    idt = models.TextField(blank=True, null=True)
    idt_pbn = models.TextField(blank=True, null=True)
    wyd_skrot = models.TextField(blank=True, null=True)
    date = models.TextField(blank=True, null=True)
    category = models.TextField(blank=True, null=True)
    details = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_pba"


class Pbd(models.Model):
    rep_f_name = models.TextField(blank=True, null=True)
    field_ignore_me = models.TextField(
        db_column="_ignore_me", blank=True, null=True
    )  # Field renamed because it started with '_'.

    class Meta:
        managed = False
        db_table = "import_dbf_pbd"


class Rtf(models.Model):
    idt = models.TextField(blank=True, null=True)
    lp = models.TextField(blank=True, null=True)
    len = models.TextField(blank=True, null=True)
    rtf = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_rtf"


class S_B(models.Model):
    idt_sci = models.TextField(blank=True, null=True)
    idt = models.ForeignKey("import_dbf.Bib", db_column="idt", on_delete=DO_NOTHING)
    cit = models.TextField(blank=True, null=True)
    doi = models.TextField(blank=True, null=True)
    del_field = models.TextField(
        db_column="del", blank=True, null=True
    )  # Field renamed because it was a Python reserved word.
    redaktor = models.TextField(blank=True, null=True)
    dat_akt = models.TextField(blank=True, null=True)
    autocyt = models.TextField(blank=True, null=True)
    ut = models.TextField(blank=True, null=True)
    ut0 = models.TextField(blank=True, null=True)
    uwagi = models.TextField(blank=True, null=True)
    field_ignore_me = models.TextField(
        db_column="_ignore_me", blank=True, null=True
    )  # Field renamed because it started with '_'.

    class Meta:
        managed = False
        db_table = "import_dbf_s_b"


class Wsy(models.Model):
    idt_wsy = models.TextField(primary_key=True)
    skrot = models.TextField(blank=True, null=True)
    nazwa = models.TextField(blank=True, null=True)
    wsp = models.TextField(blank=True, null=True)
    field_ignore_me = models.TextField(
        db_column="_ignore_me", blank=True, null=True
    )  # Field renamed because it started with '_'.

    class Meta:
        managed = False
        db_table = "import_dbf_wsy"


class Ixb(models.Model):
    idt_bazy = models.TextField(primary_key=True)
    baza = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_ixb"
        verbose_name = "zaimportowana baza"
        verbose_name_plural = "zaimportowane bazy"

    def __str__(self):
        return self.baza


class Lis(models.Model):
    rok = models.TextField(blank=True, null=True)
    kategoria = models.TextField(blank=True, null=True)
    numer = models.TextField(blank=True, null=True)
    tytul = models.TextField(blank=True, null=True)
    issn = models.TextField(blank=True, null=True)
    eissn = models.TextField(blank=True, null=True)
    punkty = models.TextField(blank=True, null=True)
    sobowtor = models.TextField(blank=True, null=True)
    errissn = models.TextField(blank=True, null=True)
    dblissn = models.TextField(blank=True, null=True)
    dbltitul = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_lis"
        verbose_name = "zaimportowana lista wydawców"
        verbose_name_plural = "zaimportowane listy wydawców"


class B_L(models.Model):
    idt = (
        models.IntegerField()
    )  # ForeignKey("import_dbf.Bib", on_delete=DO_NOTHING, db_column='idt')
    idt_l = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_b_l"


class Ext(models.Model):
    cont = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_ext"


class J_H(models.Model):
    idt_jed_f = models.ForeignKey(
        "import_dbf.Jed",
        on_delete=DO_NOTHING,
        db_column="idt_jed_f",
        related_name="jed_f",
    )
    idt_jed_t = models.ForeignKey(
        "import_dbf.Jed",
        on_delete=DO_NOTHING,
        db_column="idt_jed_t",
        related_name="jed_t",
    )
    rok = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_j_h"
        verbose_name = "zaimportowany rekord historii jednostek"
        verbose_name_plural = "zaimportowane rekordy historii jednostek"


class Kbr(models.Model):
    idt_kbr = models.TextField(primary_key=True)
    skrot = models.TextField(blank=True, null=True)
    nazwa = models.TextField(blank=True, null=True)
    to_print = models.TextField(blank=True, null=True)
    to_print2 = models.TextField(blank=True, null=True)
    to_print3 = models.TextField(blank=True, null=True)
    to_print4 = models.TextField(blank=True, null=True)
    to_print5 = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_kbr"
        verbose_name = "zaimportowany rekord KBR"
        verbose_name_plural = "zaimportowane rekordy KBR"

    def __str__(self):
        return self.nazwa


class Pbb(models.Model):
    rep_f_name = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "import_dbf_pbb"


__all__ = [
    "Bib",
    "Aut",
    "Jed",
    "B_A",
    "Poz",
    "B_U",
    "Usi",
    "Ses",
    "Wx2",
    "Ixn",
    "B_B",
    "B_N",
    "Dys",
    "Ixe",
    "Jer",
    "Kad",
    "Loc",
    "Pbc",
    "Pub",
    "Sci",
    "Sys",
    "Wsx",
    "Wyd",
    "Ldy",
    "B_E",
    "B_P",
    "Ixp",
    "Jez",
    "Kbn",
    "Pba",
    "Pbd",
    "Rtf",
    "S_B",
    "Wsy",
    "Ixb",
    "Lis",
    "B_L",
    "Ext",
    "J_H",
    "Kbr",
    "Pbb",
]



from django.urls import path

from . import views

urlpatterns = [
    path(
        "nowe_zgloszenie/",
        views.Zgloszenie_PublikacjiWizard.as_view(),
        name="nowe_zgloszenie",
    ),
    path(
        "edycja_zgloszenia/<uuid:kod_do_edycji>/",
        views.Zgloszenie_PublikacjiWizard.as_view(),
        name="edycja_zgloszenia",
    ),
    path("sukces/", views.Sukces.as_view()),
]



# Generated by Django 2.1.10 on 2019-10-21 18:08

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0185_auto_20191021_2008'),
    ]

    operations = [
        migrations.AlterField(
            model_name='autor',
            name='orcid',
            field=models.CharField(blank=True, db_index=True, help_text='Open Researcher and Contributor ID, vide http://www.orcid.org', max_length=19, null=True, unique=True, validators=[django.core.validators.RegexValidator(code='orcid_invalid_format', message='Identyfikator ORCID to 4 grupy po 4 cyfry w każdej, oddzielone myślnikami', regex='^\\d\\d\\d\\d-\\d\\d\\d\\d-\\d\\d\\d\\d-\\d\\d\\d(\\d|X)$')], verbose_name='Identyfikator ORCID'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0050_usun_model_historyczny'),
    ]

    operations = [
        migrations.AddField(
            model_name='patent_autor',
            name='zatrudniony',
            field=models.BooleanField(default=False, help_text=b'Pracownik jednostki podanej w afiliacji'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle_autor',
            name='zatrudniony',
            field=models.BooleanField(default=False, help_text=b'Pracownik jednostki podanej w afiliacji'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte_autor',
            name='zatrudniony',
            field=models.BooleanField(default=False, help_text=b'Pracownik jednostki podanej w afiliacji'),
        ),
    ]



import pytest
from model_bakery import baker

from bpp.models import Praca_Doktorska
from bpp.models.autor import Autor
from bpp.models.cache import Autorzy, Rekord
from bpp.models.openaccess import (
    Czas_Udostepnienia_OpenAccess,
    Licencja_OpenAccess,
    Wersja_Tekstu_OpenAccess,
)
from bpp.models.struktura import Jednostka
from bpp.models.system import Charakter_Formalny, Jezyk, Typ_KBN, Typ_Odpowiedzialnosci
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle, Wydawnictwo_Ciagle_Autor
from bpp.models.zrodlo import Zrodlo


def test_opis_bibliograficzny_wydawnictwo_ciagle(
    transactional_db, standard_data, denorms, autor_jan_kowalski, jednostka
):
    wc = baker.make(Wydawnictwo_Ciagle, szczegoly="Sz", uwagi="U")

    rekord = Rekord.objects.all()[0]

    assert wc.opis_bibliograficzny() == rekord.opis_bibliograficzny_cache

    wc.dodaj_autora(autor_jan_kowalski, jednostka)

    assert wc.opis_bibliograficzny() != rekord.opis_bibliograficzny_cache
    denorms.flush()
    assert wc.opis_bibliograficzny() != rekord.opis_bibliograficzny_cache


def test_opis_bibliograficzny_praca_doktorska(
    standard_data, autor_jan_kowalski, jednostka, denorms
):
    wc = baker.make(
        Praca_Doktorska,
        szczegoly="Sz",
        uwagi="U",
        autor=autor_jan_kowalski,
        jednostka=jednostka,
    )

    rekord_opis = Rekord.objects.all()[0].opis_bibliograficzny_cache
    wc_opis = wc.opis_bibliograficzny()
    assert rekord_opis == wc_opis


def test_kasowanie(db, standard_data):
    assert Rekord.objects.count() == 0

    wc = baker.make(Wydawnictwo_Ciagle)
    assert Rekord.objects.count() == 1

    wc.delete()
    assert Rekord.objects.count() == 0


def test_opis_bibliograficzny_dependent(standard_data, denorms):
    """Stwórz i skasuj Wydawnictwo_Ciagle_Autor i sprawdź, jak to
    wpłynie na opis."""

    c = baker.make(
        Wydawnictwo_Ciagle, tytul_oryginalny="Test", szczegoly="sz", uwagi="u"
    )
    assert "KOWALSKI" not in c.opis_bibliograficzny()
    assert "KOWALSKI" not in Rekord.objects.first().opis_bibliograficzny_cache

    a = baker.make(Autor, imiona="Jan", nazwisko="Kowalski")
    j = baker.make(Jednostka)
    wca = c.dodaj_autora(a, j)

    denorms.flush()

    assert "KOWALSKI" in c.opis_bibliograficzny()
    assert "KOWALSKI" in Rekord.objects.first().opis_bibliograficzny_cache

    wca.delete()

    denorms.flush()

    assert "KOWALSKI" not in c.opis_bibliograficzny()
    assert "KOWALSKI" not in Rekord.objects.first().opis_bibliograficzny_cache


def test_opis_bibliograficzny_zrodlo(standard_data, denorms):
    """Zmień nazwę źródła i sprawdź, jak to wpłynie na opis."""

    z = baker.make(Zrodlo, nazwa="OMG", skrot="wutlolski")
    c = baker.make(Wydawnictwo_Ciagle, szczegoly="SZ", uwagi="U", zrodlo=z)

    assert "wutlolski" in c.opis_bibliograficzny()
    assert "wutlolski" in Rekord.objects.first().opis_bibliograficzny_cache

    z.nazwa = "LOL"
    z.skrot = "FOKA"
    z.save()

    denorms.flush()

    assert "FOKA" in c.opis_bibliograficzny()
    assert "FOKA" in Rekord.objects.first().opis_bibliograficzny_cache

    z.nazwa = "LOL 2"
    z.skrot = "foka 2"
    z.save()

    denorms.flush()

    assert "foka 2" in c.opis_bibliograficzny()
    assert "foka 2" in Rekord.objects.first().opis_bibliograficzny_cache


@pytest.mark.django_db
def test_post_save_cache(doktorat):
    assert Rekord.objects.all().count() == 1

    doktorat.tytul = "zmieniono"
    doktorat.save()

    assert Rekord.objects.get_original(doktorat).tytul == "zmieniono"


def test_deletion_cache(doktorat):
    assert Rekord.objects.all().count() == 1

    doktorat.delete()
    assert Rekord.objects.all().count() == 0


@pytest.mark.django_db
def test_wca_delete_cache(wydawnictwo_ciagle_z_dwoma_autorami, denorms):
    """Czy skasowanie obiektu Wydawnictwo_Ciagle_Autor zmieni opis
    wydawnictwa ciągłego w Rekordy materialized view?"""

    assert Rekord.objects.all().count() == 1
    assert Wydawnictwo_Ciagle_Autor.objects.all().count() == 2

    denorms.flush()

    r = Rekord.objects.all()[0]
    assert "NOWAK" in r.opis_bibliograficzny_cache
    assert "KOWALSKI" in r.opis_bibliograficzny_cache

    Wydawnictwo_Ciagle_Autor.objects.all()[0].delete()
    aca = Wydawnictwo_Ciagle_Autor.objects.all()[0]
    aca.delete()

    denorms.flush()

    assert Autorzy.objects.filter_rekord(aca).count() == 0
    assert Rekord.objects.all().count() == 1

    r = Rekord.objects.all()[0]
    assert "NOWAK" not in r.opis_bibliograficzny_cache
    assert "KOWALSKI" not in r.opis_bibliograficzny_cache


@pytest.mark.django_db
def test_caching_kasowanie_autorow(wydawnictwo_ciagle_z_dwoma_autorami):
    for wca in Wydawnictwo_Ciagle_Autor.objects.all().only("autor"):
        wca.autor.delete()

    assert Wydawnictwo_Ciagle_Autor.objects.count() == 0
    assert Rekord.objects.all().count() == 1

    r = Rekord.objects.all()[0]
    assert "NOWAK" not in r.opis_bibliograficzny_cache
    assert "KOWALSKI" not in r.opis_bibliograficzny_cache


@pytest.mark.django_db
def test_caching_kasowanie_typu_odpowiedzialnosci_autorow(
    wydawnictwo_ciagle_z_dwoma_autorami,
):
    assert (
        Wydawnictwo_Ciagle_Autor.objects.filter(
            rekord=wydawnictwo_ciagle_z_dwoma_autorami
        ).count()
        == 2
    )

    Typ_Odpowiedzialnosci.objects.all().delete()

    assert Wydawnictwo_Ciagle_Autor.objects.count() == 0
    assert Rekord.objects.all().count() == 1

    r = Rekord.objects.all()[0]
    assert "NOWAK" not in r.opis_bibliograficzny_cache
    assert "KOWALSKI" not in r.opis_bibliograficzny_cache


@pytest.mark.django_db
def test_caching_kasowanie_zrodla(
    denorms,
    wydawnictwo_ciagle_z_dwoma_autorami,
):
    assert Zrodlo.objects.all().count() == 1

    z = Zrodlo.objects.all()[0]
    z.delete()  # WCA ma zrodlo.on_delete=SET_NULL

    denorms.flush()
    assert Rekord.objects.all().count() == 1
    assert Wydawnictwo_Ciagle.objects.all().count() == 1

    r = Rekord.objects.all()[0]
    assert "NOWAK" in r.opis_bibliograficzny_cache
    assert "KOWALSKI" in r.opis_bibliograficzny_cache
    assert z.skrot not in r.opis_bibliograficzny_cache
    assert "None" not in r.opis_bibliograficzny_cache


@pytest.mark.django_db
def test_caching_kasowanie_jezyka(wydawnictwo_ciagle_z_dwoma_autorami):
    xng = Jezyk.objects.create(skrot="xng.", nazwa="taki", pk=500)
    wydawnictwo_ciagle_z_dwoma_autorami.jezyk = xng
    wydawnictwo_ciagle_z_dwoma_autorami.save()

    assert Rekord.objects.all().count() == 1
    xng.delete()

    assert Rekord.objects.all().count() == 0


@pytest.mark.django_db
@pytest.mark.parametrize(
    "attrname, klass, skrot, ma_zostac",
    [
        ("typ_kbn", Typ_KBN, "PO", 0),
        ("charakter_formalny", Charakter_Formalny, "PAT", 0),
        ("openaccess_wersja_tekstu", Wersja_Tekstu_OpenAccess, "FINAL_AUTHOR", 1),
        ("openaccess_licencja", Licencja_OpenAccess, "CC-BY-ND", 1),
        (
            "openaccess_czas_publikacji",
            Czas_Udostepnienia_OpenAccess,
            "AT_PUBLICATION",
            1,
        ),
    ],
)
def test_usuwanie_powiazanych_vs_rekord(
    wydawnictwo_ciagle_z_dwoma_autorami,
    patent,
    standard_data,
    openaccess_data,
    attrname,
    klass,
    skrot,
    ma_zostac,
):

    o = klass.objects.get(skrot=skrot)
    setattr(wydawnictwo_ciagle_z_dwoma_autorami, attrname, o)
    wydawnictwo_ciagle_z_dwoma_autorami.save()

    setattr(patent, attrname, o)
    patent.save()

    assert Rekord.objects.all().count() == 2
    o.delete()
    assert Rekord.objects.all().count() == ma_zostac


@pytest.mark.django_db
def test_caching_kasowanie_typu_kbn(wydawnictwo_ciagle_z_dwoma_autorami, standard_data):
    tk = Typ_KBN.objects.all().first()

    wydawnictwo_ciagle_z_dwoma_autorami.typ_kbn = tk
    wydawnictwo_ciagle_z_dwoma_autorami.save()

    assert Rekord.objects.all().count() == 1
    tk.delete()

    assert Rekord.objects.all().count() == 0


@pytest.mark.django_db
def test_caching_kasowanie_charakteru_formalnego(
    wydawnictwo_ciagle_z_dwoma_autorami,
    patent,
    autor_jan_kowalski,
    jednostka,
    standard_data,
):
    patent.dodaj_autora(autor_jan_kowalski, jednostka)

    cf = Charakter_Formalny.objects.all().first()

    wydawnictwo_ciagle_z_dwoma_autorami.charakter_formalny = cf
    wydawnictwo_ciagle_z_dwoma_autorami.save()

    assert Rekord.objects.all().count() == 2
    Charakter_Formalny.objects.all().delete()

    assert Rekord.objects.all().count() == 0


@pytest.mark.django_db
def test_caching_kasowanie_wydzialu(
    autor_jan_kowalski, jednostka, wydzial, wydawnictwo_ciagle, typy_odpowiedzialnosci
):
    assert jednostka.wydzial == wydzial

    wydawnictwo_ciagle.dodaj_autora(autor_jan_kowalski, jednostka)

    assert Rekord.objects.all().count() == 1
    assert Jednostka.objects.all().count() == 1
    wydzial.delete()

    assert Rekord.objects.all().count() == 1
    assert Rekord.objects.all()[0].original.autorzy.all().count() == 0
    assert Jednostka.objects.all().count() == 0


@pytest.mark.django_db
def test_caching_kasowanie_uczelni(
    autor_jan_kowalski,
    jednostka,
    wydzial,
    uczelnia,
    wydawnictwo_ciagle,
    typy_odpowiedzialnosci,
):
    assert wydzial.uczelnia == uczelnia
    assert jednostka.wydzial == wydzial
    wydawnictwo_ciagle.dodaj_autora(autor_jan_kowalski, jednostka)

    assert Rekord.objects.all().count() == 1
    assert Jednostka.objects.all().count() == 1
    uczelnia.delete()

    assert Rekord.objects.all().count() == 1
    assert Rekord.objects.all()[0].original.autorzy.all().count() == 0
    assert Jednostka.objects.all().count() == 0


@pytest.mark.django_db
def test_caching_full_refresh(wydawnictwo_ciagle_z_dwoma_autorami, denorms):
    assert Rekord.objects.all().count() == 1
    denorms.rebuildall()
    assert Rekord.objects.all().count() == 1


@pytest.mark.django_db
def test_caching_kolejnosc(wydawnictwo_ciagle_z_dwoma_autorami, denorms):

    a = list(Wydawnictwo_Ciagle_Autor.objects.all().order_by("kolejnosc"))
    assert len(a) == 2

    denorms.flush()

    x = Rekord.objects.get_original(wydawnictwo_ciagle_z_dwoma_autorami)
    assert "[AUT.] KOWALSKI JAN, NOWAK JAN" in x.opis_bibliograficzny_cache

    k = a[0].kolejnosc
    a[0].kolejnosc = a[1].kolejnosc
    a[1].kolejnosc = k

    a[0].save()
    a[1].save()

    denorms.flush()
    x = Rekord.objects.get_original(wydawnictwo_ciagle_z_dwoma_autorami)
    assert "[AUT.] NOWAK JAN, KOWALSKI JAN" in x.opis_bibliograficzny_cache


@pytest.mark.django_db
def test_rekord_describe_content_type(wydawnictwo_zwarte):
    assert "wydawnictwo" in Rekord.objects.first().describe_content_type


@pytest.mark.django_db
def test_Rekord_get_absolute_url(wydawnictwo_zwarte):
    assert Rekord.objects.first().get_absolute_url().startswith("/")


def test_aktualizacja_rekordu_autora(typy_odpowiedzialnosci, denorms):
    w = baker.make(Wydawnictwo_Ciagle)

    a = baker.make(Autor)
    b = baker.make(Autor)

    j = baker.make(Jednostka)
    r = w.dodaj_autora(a, j, zapisany_jako="Test")

    assert b.pk not in Rekord.objects.all().first().autorzy.all().values_list(
        "autor", flat=True
    )

    r.autor = b
    r.save()

    denorms.flush()

    # czy cache jest odświeżone
    assert b.pk in Rekord.objects.all().first().autorzy.all().values_list(
        "autor", flat=True
    )


@pytest.mark.django_db
def test_prace_autora_z_afiliowanych_jednostek(typy_odpowiedzialnosci):
    a1 = baker.make(Autor, nazwisko="X", imiona="X")
    a2 = baker.make(Autor, nazwisko="Y", imiona="Y")

    nasza = baker.make(Jednostka, skupia_pracownikow=True)
    obca = baker.make(Jednostka, skupia_pracownikow=False)

    wc1 = baker.make(Wydawnictwo_Ciagle, impact_factor=10, rok=2017)
    wc2 = baker.make(Wydawnictwo_Ciagle, impact_factor=10, rok=2017)

    wc1.dodaj_autora(a1, nasza)
    wc1.dodaj_autora(a2, nasza)

    wc2.dodaj_autora(a1, obca, afiliuje=False)
    wc2.dodaj_autora(a2, nasza)

    assert Rekord.objects.prace_autora(a1).count() == 2
    assert Rekord.objects.prace_autora(a2).count() == 2

    assert Rekord.objects.prace_autora_z_afiliowanych_jednostek(a1).count() == 1
    assert Rekord.objects.prace_autora_z_afiliowanych_jednostek(a2).count() == 2


@pytest.mark.django_db
def test_rebuild_ciagle(
    django_assert_max_num_queries, wydawnictwo_ciagle_z_dwoma_autorami, denorms
):
    with django_assert_max_num_queries(40):
        denorms.rebuildall("Wydawnictwo_Ciagle")


@pytest.mark.django_db
def test_rebuild_zwarte(
    django_assert_max_num_queries, wydawnictwo_zwarte_z_autorem, denorms
):
    with django_assert_max_num_queries(45):
        denorms.rebuildall("Wydawnictwo_Zwarte")


@pytest.mark.django_db
def test_rebuild_patent(django_assert_max_num_queries, patent, denorms):
    with django_assert_max_num_queries(40):
        denorms.rebuildall("Patent")



from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support.expected_conditions import alert_is_present

from bpp.const import CHARAKTER_OGOLNY_KSIAZKA, TO_AUTOR

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

import pytest
from django.db import transaction
from model_bakery import baker
from selenium.webdriver.support.wait import WebDriverWait

from bpp.models import (
    Autor,
    Charakter_Formalny,
    Jednostka,
    Typ_Odpowiedzialnosci,
    Uczelnia,
    Wydawnictwo_Ciagle,
)
from bpp.models.patent import Patent
from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte
from bpp.models.zrodlo import Punktacja_Zrodla
from bpp.tests import (
    add_extra_autor_inline,
    any_autor,
    any_ciagle,
    any_jednostka,
    assertPopupContains,
    proper_click_by_id,
    proper_click_element,
)
from bpp.tests.util import (
    CURRENT_YEAR,
    any_zrodlo,
    select_select2_autocomplete,
    select_select2_clear_selection,
    show_element,
)

from django_bpp.selenium_util import (
    LONG_WAIT_TIME,
    SHORT_WAIT_TIME,
    wait_for,
    wait_for_page_load,
)

ID = "id_tytul_oryginalny"

pytestmark = [pytest.mark.slow, pytest.mark.selenium]


@pytest.mark.django_db(transaction=True)
@pytest.mark.parametrize("url", ["wydawnictwo_ciagle", "wydawnictwo_zwarte"])
def test_uzupelnij_strona_tom_nr_zeszytu(url, admin_browser, asgi_live_server):
    url = reverse("admin:bpp_%s_add" % url)
    with wait_for_page_load(admin_browser):
        admin_browser.visit(asgi_live_server.url + url)

    WebDriverWait(admin_browser, SHORT_WAIT_TIME).until(
        lambda browser: not admin_browser.find_by_name("informacje").is_empty()
    )
    try:
        WebDriverWait(admin_browser, LONG_WAIT_TIME).until(
            lambda browser: not admin_browser.find_by_id("id_strony_get").is_empty()
        )
    except TimeoutException as e:
        raise e

    admin_browser.find_by_name("informacje").type("1993 vol. 5 z. 1")
    admin_browser.find_by_name("szczegoly").type("s. 4-3")

    elem = admin_browser.find_by_id("id_strony_get")
    show_element(admin_browser, elem)
    elem.click()

    WebDriverWait(admin_browser, SHORT_WAIT_TIME).until(
        lambda browser: browser.find_by_name("tom").value != ""
    )

    assert admin_browser.find_by_name("strony").value == "4-3"
    assert admin_browser.find_by_name("tom").value == "5"

    if url == "wydawnictwo_ciagle":
        assert admin_browser.find_by_name("nr_zeszytu").value == "1"


def test_liczba_znakow_wydawniczych_liczba_arkuszy_wydawniczych(
    admin_browser, asgi_live_server
):
    url = reverse("admin:bpp_wydawnictwo_zwarte_add")
    with wait_for_page_load(admin_browser):
        admin_browser.visit(asgi_live_server.url + url)

    try:
        WebDriverWait(admin_browser, SHORT_WAIT_TIME).until(
            lambda browser: not admin_browser.find_by_id(
                "id_liczba_arkuszy_wydawniczych"
            ).is_empty()
        )
    except TimeoutException as e:
        raise e

    admin_browser.execute_script(
        "django.jQuery('#id_liczba_znakow_wydawniczych').val('40000').change()"
    )
    assert admin_browser.find_by_id("id_liczba_arkuszy_wydawniczych").value == "1.00"

    admin_browser.execute_script(
        "django.jQuery('#id_liczba_arkuszy_wydawniczych').val('0.5').change()"
    )
    assert admin_browser.find_by_id("id_liczba_znakow_wydawniczych").value == "20000"


@pytest.mark.django_db(transaction=True)
def test_automatycznie_uzupelnij_punkty(admin_browser, asgi_live_server):
    url = reverse("admin:bpp_wydawnictwo_ciagle_add")

    with wait_for_page_load(admin_browser):
        admin_browser.visit(asgi_live_server.url + url)

    any_zrodlo(nazwa="FOO BAR")

    elem = admin_browser.find_by_id("id_wypelnij_pola_punktacji_button")
    show_element(admin_browser, elem)
    elem.click()
    assertPopupContains(admin_browser, "Najpierw wybierz jakie")

    select_select2_autocomplete(admin_browser, "id_zrodlo", "FOO")

    proper_click_by_id(admin_browser, "id_wypelnij_pola_punktacji_button")
    assertPopupContains(admin_browser, "Uzupełnij pole")

    admin_browser.execute_script("window.onbeforeunload = function(e) {};")


def trigger_event(elem, event):
    elem.parent.driver.execute_script(
        "return django.jQuery(arguments[0]).trigger(arguments[1]).get();",
        "#" + elem["id"],
        event,
    )


def test_admin_uzupelnij_punkty(admin_browser, asgi_live_server, denorms):
    z = any_zrodlo(nazwa="WTF LOL")

    kw = dict(zrodlo=z)
    f = Punktacja_Zrodla.objects.create
    f(impact_factor=10.1, punkty_kbn=10.2, rok=CURRENT_YEAR, **kw)
    f(impact_factor=11.1, punkty_kbn=11.2, rok=CURRENT_YEAR + 1, **kw)

    c = any_ciagle(zrodlo=z, impact_factor=5, punkty_kbn=5)

    url = reverse("admin:bpp_wydawnictwo_ciagle_change", args=(c.pk,))
    with wait_for_page_load(admin_browser):
        admin_browser.visit(asgi_live_server.url + url)

    rok = admin_browser.find_by_id("id_rok")
    punkty_kbn = admin_browser.find_by_id("id_punkty_kbn")

    assert rok.value == str(CURRENT_YEAR)
    assert punkty_kbn.value == "5.00"

    button = admin_browser.find_by_id("id_wypelnij_pola_punktacji_button")

    proper_click_element(admin_browser, button)

    wait_for(lambda: button.value == "Wypełniona!")
    wait_for(lambda: punkty_kbn.value == "10.20")

    trigger_event(rok, "change")
    # Po zmianie roku LUB źródła przycisk ma się zmienić
    wait_for(lambda: button.value == "Wypełnij pola punktacji")

    # Zwiększymy rok o 1 i sprawdzimy, czy zmieni się punktacja
    admin_browser.fill("rok", str(CURRENT_YEAR + 1))

    button = admin_browser.find_by_id("id_wypelnij_pola_punktacji_button")
    proper_click_element(admin_browser, button)
    wait_for(lambda: punkty_kbn.value == "11.20")

    # Teraz usuniemy źródło i sprawdzimy, czy przycisk zmieni nazwę
    assert button.value == "Wypełniona!"

    select_select2_clear_selection(admin_browser, "id_zrodlo")
    WebDriverWait(admin_browser.driver, SHORT_WAIT_TIME).until(
        lambda browser: admin_browser.find_by_id(
            "id_wypelnij_pola_punktacji_button"
        ).value
        == "Wypełnij pola punktacji"
    )

    admin_browser.execute_script("window.onbeforeunload = function(e) {};")


def test_upload_punkty(admin_browser, asgi_live_server):
    any_zrodlo(nazwa="WTF LOL")

    url = reverse("admin:bpp_wydawnictwo_ciagle_add")
    admin_browser.visit(asgi_live_server.url + url)

    select_select2_autocomplete(admin_browser, "id_zrodlo", "WTF")

    rok = admin_browser.find_by_id("id_rok")
    show_element(admin_browser, rok)
    admin_browser.fill("rok", str(CURRENT_YEAR))

    show_element(admin_browser, admin_browser.find_by_id("id_impact_factor"))
    admin_browser.fill("impact_factor", "1")

    elem = admin_browser.find_by_id("id_dodaj_punktacje_do_zrodla_button")
    show_element(admin_browser, elem)
    elem.click()

    WebDriverWait(admin_browser.driver, SHORT_WAIT_TIME).until(
        lambda browser: Punktacja_Zrodla.objects.count() == 1
    )
    assert Punktacja_Zrodla.objects.all()[0].impact_factor == 1

    admin_browser.fill("impact_factor", "2")
    proper_click_by_id(admin_browser, "id_dodaj_punktacje_do_zrodla_button")

    WebDriverWait(admin_browser.driver, SHORT_WAIT_TIME).until(alert_is_present())
    assertPopupContains(admin_browser, "Punktacja dla tego roku już istnieje")
    WebDriverWait(admin_browser.driver, SHORT_WAIT_TIME).until(
        lambda browser: not alert_is_present()(browser)
    )

    WebDriverWait(admin_browser.driver, SHORT_WAIT_TIME).until(
        lambda browser: Punktacja_Zrodla.objects.all()[0].impact_factor == 2
    )

    admin_browser.execute_script("window.onbeforeunload = function(e) {};")


@pytest.fixture
def autorform_jednostka(db):
    with transaction.atomic():
        a = any_autor(nazwisko="KOWALSKI", imiona="Jan Sebastian")
        j = any_jednostka(nazwa="WTF LOL")
        j.dodaj_autora(a)
    return j


@pytest.fixture
def autorform_browser(admin_browser, db, asgi_live_server):
    url = reverse("admin:bpp_wydawnictwo_ciagle_add")
    with wait_for_page_load(admin_browser):
        admin_browser.visit(asgi_live_server.url + url)

    admin_browser.execute_script("window.onbeforeunload = function(e) {};")
    return admin_browser


def test_autorform_uzupelnianie_jednostki(autorform_browser, autorform_jednostka):
    add_extra_autor_inline(autorform_browser)

    select_select2_autocomplete(autorform_browser, "id_autorzy_set-0-autor", "KOWALSKI")

    autorform_browser.find_by_id("id_autorzy_set-0-jednostka")
    WebDriverWait(autorform_browser, SHORT_WAIT_TIME).until(
        lambda browser: autorform_browser.find_by_id("id_autorzy_set-0-jednostka").value
        == str(autorform_jednostka.pk)
    )


def find_autocomplete_widget(browser, id):
    for elem in browser.find_by_css(".yourlabs-autocomplete"):
        try:
            dii = elem["data-input-id"]
        except KeyError:
            continue

        if dii == id:
            return elem


def test_autorform_kasowanie_autora(autorform_browser, autorform_jednostka):
    # kliknij "dodaj powiazanie autor-wydawnictwo"
    add_extra_autor_inline(autorform_browser)

    # uzupełnij autora
    select_select2_autocomplete(autorform_browser, "id_autorzy_set-0-autor", "KOW")

    def jednostka_ustawiona():
        jed = autorform_browser.find_by_id("id_autorzy_set-0-jednostka")
        if jed.value != "":
            return jed.value

    wait_for(jednostka_ustawiona)

    # Jednostka ustawiona. Usuń autora:
    select_select2_clear_selection(autorform_browser, "id_autorzy_set-0-autor")

    # jednostka nie jest wybrana
    try:
        WebDriverWait(autorform_browser.driver, SHORT_WAIT_TIME).until(
            lambda browser: autorform_browser.find_by_id(
                "id_autorzy_set-0-jednostka"
            ).value.find("\n")
            != -1
        )
    except TimeoutException as e:
        raise e
    #  assert jed.value.find("\n") != -1

    autorform_browser.execute_script("window.onbeforeunload = function(e) {};")


def test_bug_on_user_add(admin_browser, asgi_live_server):
    admin_browser.visit(asgi_live_server.url + reverse("admin:bpp_bppuser_add"))
    admin_browser.fill("username", "as")
    admin_browser.fill("password1", "as")
    admin_browser.fill("password2", "as")
    with wait_for_page_load(admin_browser):
        admin_browser.find_by_name("_continue").click()

    admin_browser.wait_for_condition(lambda browser: "Zmień użytkownik" in browser.html)


@pytest.mark.django_db
def test_admin_wydawnictwo_zwarte_uzupelnij_rok(
    wydawnictwo_zwarte, admin_browser, asgi_live_server, transactional_db
):
    """
    :type admin_browser: splinter.driver.webdriver.remote.WebDriver
    """

    browser = admin_browser

    browser.visit(asgi_live_server.url + reverse("admin:bpp_wydawnictwo_zwarte_add"))

    rok = browser.find_by_id("id_rok")
    button = browser.find_by_id("id_rok_button")

    assert rok.value == ""

    browser.fill("miejsce_i_rok", "Lublin 2002")

    proper_click_element(browser, button)

    browser.wait_for_condition(
        lambda browser: browser.find_by_id("id_rok").value == "2002"
    )

    chf = Charakter_Formalny.objects.create(
        nazwa="charakter", skrot="ch", charakter_ogolny=CHARAKTER_OGOLNY_KSIAZKA
    )

    wydawnictwo_zwarte.rok = 1997
    wydawnictwo_zwarte.charakter_formalny = chf
    wydawnictwo_zwarte.save()

    select_select2_autocomplete(
        browser, "id_wydawnictwo_nadrzedne", "Wydawnictwo Zwarte"
    )

    browser.fill("rok", "")
    proper_click_element(browser, button)
    browser.wait_for_condition(
        lambda browser: browser.find_by_id("id_rok").value == "2002"
    )

    browser.fill("miejsce_i_rok", "")
    proper_click_element(browser, button)
    browser.wait_for_condition(
        lambda browser: browser.find_by_id("id_rok").value == "1997"
    )


def test_admin_wydawnictwo_ciagle_uzupelnij_rok(admin_browser, asgi_live_server):
    """
    :type admin_browser: splinter.driver.webdriver.remote.WebDriver
    """

    browser = admin_browser

    with wait_for_page_load(browser):
        browser.visit(
            asgi_live_server.url + reverse("admin:bpp_wydawnictwo_ciagle_add")
        )
    browser.fill("informacje", "Lublin 2002 test")
    elem = browser.find_by_id("id_rok_button")
    proper_click_element(browser, elem)

    browser.wait_for_condition(
        lambda browser: browser.find_by_id("id_rok").value == "2002"
    )

    browser.fill("informacje", "")
    proper_click_by_id(browser, "id_rok_button")
    browser.wait_for_condition(
        lambda browser: browser.find_by_id("id_rok_button").value == "Brak danych"
    )


def test_admin_wydawnictwo_ciagle_dowolnie_zapisane_nazwisko(
    admin_browser, asgi_live_server, autor_jan_kowalski
):
    """
    :type admin_browser: splinter.driver.webdriver.remote.WebDriver
    """

    browser = admin_browser

    with wait_for_page_load(browser):
        browser.visit(
            asgi_live_server.url + reverse("admin:bpp_wydawnictwo_ciagle_add")
        )

    xp1 = "/html/body/div[2]/article/div/form/div/div[1]/ul/li/a"
    wait_for(lambda: len(browser.find_by_xpath(xp1)) > 0)
    elem = browser.find_by_xpath(xp1)
    proper_click_element(browser, elem[0])

    element = browser.find_by_xpath(
        "/html/body/div[2]/article/div/form/div/div[1]/div[2]/div/a"
    )
    proper_click_element(browser, element)
    # scroll_into_view()
    wait_for(
        lambda: browser.find_by_id("id_autorzy_set-0-autor"),
        max_seconds=SHORT_WAIT_TIME,
    )

    select_select2_autocomplete(browser, "id_autorzy_set-0-autor", "Kowalski Jan")

    select_select2_autocomplete(
        browser, "id_autorzy_set-0-zapisany_jako", "Dowolny tekst"
    )

    if browser.find_by_id("id_autorzy_set-0-zapisany_jako").value != "Dowolny tekst":
        print("1")  # for debugging

    assert browser.find_by_id("id_autorzy_set-0-zapisany_jako").value == "Dowolny tekst"


@pytest.mark.parametrize("expected", [True, False])
@pytest.mark.parametrize(
    "url",
    ["wydawnictwo_ciagle", "wydawnictwo_zwarte", "patent"],
)
def test_admin_domyslnie_afiliuje_nowy_rekord(
    admin_browser,
    asgi_live_server,
    url,
    expected,
):
    # twórz nowy obiekt, nie używaj z fixtury, bo db i transactional_db
    baker.make(Uczelnia, domyslnie_afiliuje=expected)

    browser = admin_browser
    with wait_for_page_load(browser):
        browser.visit(asgi_live_server.url + reverse(f"admin:bpp_{url}_add"))

    add_extra_autor_inline(browser)

    v = browser.find_by_id("id_autorzy_set-0-afiliuje")
    assert v.checked == expected


@pytest.mark.parametrize("afiliowany", [True, False])
@pytest.mark.parametrize("expected", [True, False])
@pytest.mark.parametrize(
    "url,klasa",
    [
        ("wydawnictwo_ciagle", Wydawnictwo_Ciagle),
        ("wydawnictwo_zwarte", Wydawnictwo_Zwarte),
        ("patent", Patent),
    ],
)
# @pytest.mark.django_db(transaction=True)
def test_admin_domyslnie_afiliuje_istniejacy_rekord(
    admin_browser, asgi_live_server, url, klasa, expected, afiliowany, denorms
):
    # twórz nowy obiekt, nie używaj z fixtury, bo db i transactional_db
    baker.make(Uczelnia, domyslnie_afiliuje=expected)
    autor = baker.make(Autor, nazwisko="Kowal", imiona="Ski")
    jednostka = baker.make(Jednostka, nazwa="Lol", skrot="WT")
    wydawnictwo = baker.make(klasa, tytul_oryginalny="test")
    Typ_Odpowiedzialnosci.objects.get_or_create(
        skrot="aut.", nazwa="autor", typ_ogolny=TO_AUTOR
    )
    wa = wydawnictwo.dodaj_autora(autor, jednostka, zapisany_jako="Wutlolski")
    wa.afiliowany = afiliowany
    wa.save()

    browser = admin_browser
    with wait_for_page_load(browser):
        browser.visit(
            asgi_live_server.url
            + reverse(f"admin:bpp_{url}_change", args=(wydawnictwo.pk,))
        )

    add_extra_autor_inline(browser)

    v = browser.find_by_id("id_autorzy_set-1-afiliuje")
    assert v.checked == expected



# Generated by Django 2.1.7 on 2019-09-02 23:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0176_auto_20190903_0108'),
    ]

    operations = [
        migrations.CreateModel(
            name='Cache_Punktacja_Autora_Query',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('pkdaut', models.DecimalField(decimal_places=4, max_digits=20)),
                ('slot', models.DecimalField(decimal_places=4, max_digits=20)),
            ],
            options={
                'db_table': 'cache_punktacja_autora',
                'managed': False,
            },
        ),
    ]



import pytest

from crossref_bpp.models import CrossrefAPICache


@pytest.mark.django_db
def test_CrossrefAPICacheManager_cleanup():
    CrossrefAPICache.objects.cleanup()


@pytest.mark.django_db
def test_CrossrefAPICacheManager_get_by_doi(mocker):
    with mocker.patch.object(
        CrossrefAPICache.objects, "api_get_by_doi", return_value={"a": "b"}
    ):
        data = CrossrefAPICache.objects.get_by_doi("whatever")
    assert data["a"] == "b"


@pytest.mark.django_db
def test_CrossrefAPICacheManager_get_by_doi_no_queries(
    mocker, django_assert_max_num_queries
):
    with django_assert_max_num_queries(5):
        with mocker.patch.object(
            CrossrefAPICache.objects, "api_get_by_doi", return_value={"a": "b"}
        ):
            CrossrefAPICache.objects.get_by_doi("whatever")
            CrossrefAPICache.objects.get_by_doi("whatever")
            CrossrefAPICache.objects.get_by_doi("whatever")



# -*- encoding: utf-8 -*-
from django.http import HttpResponse
import json

from django.utils.deprecation import MiddlewareMixin


class NonHtmlDebugToolbarMiddleware(MiddlewareMixin):
    """
    The Django Debug Toolbar usually only works for views that return HTML.
    This middleware wraps any non-HTML response in HTML if the request
    has a 'debug' query parameter (e.g. http://localhost/foo?debug)
    Special handling for json (pretty printing) and
    binary data (only show data length)
    """

    @staticmethod
    def process_response(request, response):
        debug = request.GET.get('debug', 'UNSET')

        if debug != 'UNSET':
            if response['Content-Type'] == 'application/octet-stream':
                new_content = '<html><body>Binary Data, ' \
                              'Length: {}</body></html>'.format(
                    len(response.content))
                response = HttpResponse(new_content)
            elif response['Content-Type'] != 'text/html':
                content = response.content
                try:
                    json_ = json.loads(content)
                    content = json.dumps(json_, sort_keys=True, indent=2)
                except ValueError:
                    pass
                response = HttpResponse('<html><body><pre>{}'
                                        '</pre></body></html>'.format(content))

        return response



# Generated by Django 2.2.10 on 2020-03-04 23:17

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        (
            "rozbieznosci_dyscyplin",
            "0005_recreate",
        ),
        ("bpp", "0237_auto_20210223_2228"),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql(
                "0002_rok_2017_i_wyzej", app_name="rozbieznosci_dyscyplin"
            )
        )
    ]



import json
import re

from django.contrib.contenttypes.models import ContentType

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from django.db.models.query_utils import Q
from django.http import Http404, HttpResponseForbidden, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.views.generic import DetailView, ListView, RedirectView
from multiseek.logic import AND, OR
from multiseek.util import make_field
from multiseek.views import MULTISEEK_SESSION_KEY, MULTISEEK_SESSION_KEY_REMOVED

from miniblog.models import Article

from bpp.models import Autor, Jednostka, Rekord, Uczelnia, Wydzial, Zrodlo
from bpp.multiseek_registry import (
    JednostkaQueryObject,
    NazwiskoIImieQueryObject,
    RokQueryObject,
    TypRekorduObject,
    ZrodloQueryObject,
)

PUBLIKACJE = "publikacje"
STRESZCZENIA = "streszczenia"
INNE = "inne"
TYPY = [PUBLIKACJE, STRESZCZENIA, INNE]


def conditional(**kwargs):
    """A wrapper around :func:`django.views.decorators.http.condition` that
    works for methods (i.e. class-based views).
    """
    from django.views.decorators.http import condition

    from django.utils.decorators import method_decorator

    return method_decorator(condition(**kwargs))


class UczelniaView(DetailView):
    model = Uczelnia
    template_name = "browse/uczelnia.html"

    def get_context_data(self, **kwargs):
        context = {}
        if "article_slug" in self.kwargs:
            context["article"] = get_object_or_404(
                Article, slug=self.kwargs["article_slug"]
            )
        else:
            context["miniblog"] = Article.objects.filter(
                status=Article.STATUS.published
            )[:5]

        context.update(kwargs)
        return super().get_context_data(**context)


class WydzialView(DetailView):
    template_name = "browse/wydzial.html"
    model = Wydzial


class JednostkaView(DetailView):
    template_name = "browse/jednostka.html"
    model = Jednostka

    def get_context_data(self, **kwargs):
        return super().get_context_data(typy=TYPY, **kwargs)


class AutorView(DetailView):
    template_name = "browse/autor.html"
    model = Autor

    def get_context_data(self, **kwargs):
        return super().get_context_data(typy=TYPY, **kwargs)


LITERKI = "ABCDEFGHIJKLMNOPQRSTUVWYXZ"

PODWOJNE = {
    "A": ["A", "Ą", "ą"],
    "C": ["C", "Ć", "ć"],
    "E": ["E", "Ę", "ę"],
    "L": ["L", "Ł", "ł"],
    "N": ["N", "Ń", "ń"],
    "O": ["O", "Ó", "ó"],
    "Z": ["Z", "Ź", "Ż", "ź", "ż"],
}


class Browser(ListView):
    model = None
    param = None
    literka_field = None
    paginate_by = 40

    def get_search_string(self):
        return self.request.GET.get(self.param, "")

    def get_literka(self):
        return self.kwargs.get("literka")

    def get_queryset(self):
        literka = self.get_literka()
        sstr = self.get_search_string()

        if sstr:
            qry = self.model.objects.fulltext_filter(self.get_search_string())
        else:
            qry = self.model.objects.all()

        if literka:
            args = [literka]
            if literka in PODWOJNE:
                args = PODWOJNE[literka]

            qobj = Q(**{self.literka_field + "__istartswith": literka})
            for x in args[1:]:
                qobj |= Q(**{self.literka_field + "__istartswith": x})
            qry = qry.filter(qobj)  # **{self.param + "__istartswith": literka})

        return qry.distinct()

    def get_context_data(self, *args, **kw):
        return super().get_context_data(
            flt=self.request.GET.get(self.param),
            literki=LITERKI,
            wybrana=self.kwargs.pop("literka", None),
            *args,
            **kw
        )


class AutorzyView(Browser):
    template_name = "browse/autorzy.html"
    model = Autor
    param = "search"
    literka_field = "nazwisko"
    paginate_by = 252

    def get_queryset(self):
        return (
            super()
            .get_queryset()
            .only("nazwisko", "imiona", "slug", "poprzednie_nazwiska")
        )


class ZrodlaView(Browser):
    template_name = "browse/zrodla.html"
    model = Zrodlo
    param = "search"
    literka_field = "nazwa"
    paginate_by = 70

    def get_queryset(self):
        return super().get_queryset().only("nazwa", "poprzednia_nazwa", "slug")


class JednostkiView(Browser):
    template_name = "browse/jednostki.html"
    model = Jednostka
    param = "search"
    literka_field = "nazwa"
    paginate_by = 150

    def get_paginate_by(self, queryset):
        uczelnia = None

        if hasattr(self, "request") and self.request is not None:
            uczelnia = Uczelnia.objects.get_for_request(self.request)

        if uczelnia is None:
            uczelnia = Uczelnia.objects.get_default()

        if uczelnia is None:
            return self.paginate_by

        return uczelnia.ilosc_jednostek_na_strone

    def get_queryset(self):
        ordering = None

        qry = super().get_queryset().filter(widoczna=True)

        uczelnia = Uczelnia.objects.get_for_request(self.request)
        if uczelnia:
            if uczelnia.sortuj_jednostki_alfabetycznie:
                ordering = ("nazwa",)

            if uczelnia.pokazuj_tylko_jednostki_nadrzedne:
                qry = qry.filter(parent=None)

        ret = qry.only("nazwa", "slug", "wydzial").select_related("wydzial")

        if ordering:
            ret = ret.order_by(*ordering)

        return ret


class ZrodloView(DetailView):
    model = Zrodlo
    template_name = "browse/zrodlo.html"


def zrob_box(values, name, qo):
    ret = []
    po = None
    for value in values:
        ret.append(make_field(qo, qo.ops[0], value, prev_op=po))
        po = OR
    return ret


def zrob_box_z_requestu(dct, name, qo):
    if dct.get(name):
        return zrob_box(dct.getlist(name), name, qo)
    return []


def zrob_formularz(*args):
    ret = [None]

    prev_op = None

    for arg in [x for x in args if x]:

        if len(arg) > 1:
            lst = [prev_op]
            lst.extend(arg)
            ret.append(lst)
        else:
            arg[0]["prev_op"] = prev_op
            ret.append(arg[0])

        prev_op = AND

    return json.dumps({"form_data": ret})


class BuildSearch(RedirectView):
    def get_redirect_url(self, **kwargs):
        url = self.request.build_absolute_uri(reverse("multiseek:index"))
        scheme = self.request.META.get("HTTP_X_SCHEME", "").lower()
        if scheme == "https":
            url = url.replace("http://", "https://").replace("HTTP://", "HTTPS://")
        return url

    def post(self, *args, **kw):
        zrodla_box = zrob_box_z_requestu(self.request.POST, "zrodlo", ZrodloQueryObject)
        typy_box = zrob_box_z_requestu(self.request.POST, "typ", TypRekorduObject)
        lata_box = zrob_box_z_requestu(self.request.POST, "rok", RokQueryObject)
        jednostki_box = zrob_box_z_requestu(
            self.request.POST, "jednostka", JednostkaQueryObject
        )
        autorzy_box = zrob_box_z_requestu(
            self.request.POST, "autor", NazwiskoIImieQueryObject
        )

        self.request.session[MULTISEEK_SESSION_KEY] = zrob_formularz(
            zrodla_box, autorzy_box, typy_box, jednostki_box, lata_box
        )

        self.request.session["MULTISEEK_TITLE"] = self.request.POST.get(
            "suggested-title", ""
        )

        # Usuń listę ręcznie wyrzuconych rekordów, ponieważ wchodzimy na świeże
        # wyszukiwanie (#325)
        self.request.session.pop(MULTISEEK_SESSION_KEY_REMOVED, None)

        return super().post(*args, **kw)


class PracaViewMixin:
    def get(self, request, *args, **kwargs):
        self.object = self.get_object()

        if request.user.is_anonymous:
            # Jeżeli użytkownik jest anonimowy, to może obejmować go ukrywanie statusów
            uczelnia = Uczelnia.objects.get_for_request(request)

            if uczelnia is not None:
                statusy = uczelnia.ukryte_statusy("podglad")

                if self.object.status_korekty_id in statusy:
                    return HttpResponseForbidden("Brak uprawnień do rekordu")

        if (
            "slug" not in self.kwargs
            and hasattr(self.object, "slug")
            and self.object.slug
        ):
            return HttpResponseRedirect(
                reverse("bpp:browse_praca_by_slug", args=(self.object.slug,))
            )

        if (
            "slug" in self.kwargs
            and hasattr(self.object, "slug")
            and self.object.slug != self.kwargs["slug"]
        ):
            # Przy przekierowaniu z linku typu 'starytekst_32_4533'
            # (because cool URIs don't change!)
            return HttpResponseRedirect(
                reverse("bpp:browse_praca_by_slug", args=(self.object.slug,))
            )

        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)


END_NUMBER_REGEX = re.compile("(?P<content_type_id>\\d+)-(?P<object_id>\\d+)$")


class PracaViewBySlug(PracaViewMixin, DetailView):
    template_name = "browse/praca.html"
    model = Rekord

    def get_object(self):
        try:
            return Rekord.objects.get(slug=self.kwargs["slug"])
        except (Rekord.DoesNotExist, Rekord.MultipleObjectsReturned):
            # Sprawdź cyferki na końcu
            res = re.search(END_NUMBER_REGEX, self.kwargs["slug"])
            if res is not None:
                content_type_id, object_id = None, None
                try:
                    content_type_id, object_id = res.groups()
                except ValueError:
                    pass

                if content_type_id is not None and object_id is not None:
                    try:
                        obj = Rekord.objects.get(pk=[content_type_id, object_id])
                    except Rekord.DoesNotExist:
                        raise Http404
                    except Rekord.MultipleObjectsReturned:
                        raise NotImplementedError("This should never happen")

                    return obj

        raise Http404


class PracaView(PracaViewMixin, DetailView):
    template_name = "browse/praca.html"
    model = Rekord

    def get_object(self, queryset=None):
        model = self.kwargs["model"]

        try:
            int(model)
        except ValueError:
            try:
                model = ContentType.objects.get_by_natural_key("bpp", model).pk
            except ContentType.DoesNotExist:
                raise Http404

        try:
            obj = Rekord.objects.get(pk=[model, self.kwargs["pk"]])
        except Rekord.DoesNotExist:
            raise Http404

        return obj


class OldPracaView(RedirectView):
    permanent = True

    def get_redirect_url(self, *args, **kwargs):
        return reverse(
            "bpp:browse_praca",
            args=(
                ContentType.objects.get(app_label="bpp", model=self.kwargs["model"]).pk,
                self.kwargs["pk"],
            ),
        )


class RekordToPracaView(RedirectView):
    model = Rekord

    def get_redirect_url(self, *args, **kw):
        return reverse(
            "bpp:browse_praca",
            args=(self.kwargs["content_type_id"], self.kwargs["object_id"]),
        )



import pytest
from django.core.files.base import File
from model_bakery import baker

from ewaluacja2021.models import ImportMaksymalnychSlotow
from ewaluacja2021.tests.utils import curdir

from bpp.models import Dyscyplina_Naukowa


@pytest.mark.django_db
def test_ImportMaksymalnychSlotow_analizuj(autor_jan_kowalski):
    baker.make(Dyscyplina_Naukowa, nazwa="nauki farmaceutyczne")

    ims = ImportMaksymalnychSlotow.objects.create()
    ims.plik.save("test.xlsx", File(open(curdir("test_file.xlsx", __file__), "rb")))
    ims.analizuj()

    assert ims.przeanalizowany
    assert ims.wierszimportumaksymalnychslotow_set.filter(poprawny=True).count() == 1
    assert (
        ims.wierszimportumaksymalnychslotow_set.filter(wymagana_integracja=True).count()
        == 1
    )
    assert ims.wierszimportumaksymalnychslotow_set.filter(poprawny=False).count() == 1



class WydawcaPBNAdapter:
    def __init__(self, original):
        self.original = original

    def pbn_get_json(self):
        wydawca = self.original.get_toplevel()

        if wydawca.pbn_uid_id is not None:
            pbn_wydawca = wydawca.pbn_uid
            ret = {
                "objectId": pbn_wydawca.pk,
                "name": pbn_wydawca.value("object", "publisherName"),
            }
            mniswId = pbn_wydawca.value_or_none("object", "mniswId")
            if mniswId:
                ret["mniswId"] = mniswId
            return ret

        return {
            "name": wydawca.nazwa,
        }



from datetime import timedelta
from unittest.mock import Mock

import pytest
from model_bakery import baker

from celeryui.models import Report

from django.utils import timezone

from bpp.models import Wydawnictwo_Ciagle
from bpp.tasks import _zaktualizuj_liczbe_cytowan, remove_old_report_files


@pytest.mark.django_db
def test_remove_old_report_files():
    baker.make(Report)
    r = baker.make(Report)
    r.started_on = timezone.now() - timedelta(days=20)
    r.save()

    assert Report.objects.all().count() == 2

    remove_old_report_files()

    assert Report.objects.all().count() == 1


@pytest.mark.django_db
def test_zaktualizuj_liczbe_cytowan(uczelnia, wydawnictwo_ciagle, mocker):

    m = Mock()
    m.query_multiple = Mock(
        return_value=[{wydawnictwo_ciagle.pk: {"timesCited": "31337"}}]
    )

    mocker.patch("bpp.models.struktura.Uczelnia.wosclient", return_value=m)

    _zaktualizuj_liczbe_cytowan(
        [
            Wydawnictwo_Ciagle,
        ]
    )

    wydawnictwo_ciagle.refresh_from_db()
    assert wydawnictwo_ciagle.liczba_cytowan == 31337



from django.core.management import BaseCommand

from tee import const


class Command(BaseCommand):
    help = const.DONT_CALL

    def handle(self, *args, **options):

        self.stderr.write("wrote to stderr")
        self.stdout.write("wrote to stdout")
        print("Used print()")
        print("Used print() with file=self.stderr", file=self.stderr)



from pbn_api.admin.base import BasePBNAPIAdmin
from pbn_api.admin.filters import (
    OdpowiednikPublikacjiInstytucjiAutorWBPPFilter,
    OdpowiednikPublikacjiInstytucjiPublikacjaWBPPFilter,
)
from pbn_api.models import PublikacjaInstytucji

from django.contrib import admin

from bpp.models import Rekord


@admin.register(PublikacjaInstytucji)
class PublikacjaInstytucjiAdmin(BasePBNAPIAdmin):
    list_per_page = 25
    actions = None
    autocomplete_fields = [
        "insPersonId",
        "institutionId",
        "publicationId",
    ]
    list_select_related = ["insPersonId", "institutionId", "publicationId"]
    list_filter = [
        "userType",
        "publicationType",
        OdpowiednikPublikacjiInstytucjiPublikacjaWBPPFilter,
        OdpowiednikPublikacjiInstytucjiAutorWBPPFilter,
    ]
    list_display = [
        "publicationId",
        "odpowiednik_publikacji_w_bpp",
        "publicationType",
        "insPersonId",
        "odpowiednik_osoby_w_bpp",
        "userType",
    ]
    readonly_fields = [
        "insPersonId",
        "institutionId",
        "publicationId",
        "publicationType",
        "userType",
        "publicationVersion",
        "publicationYear",
    ]

    fields = readonly_fields + ["snapshot"]

    def odpowiednik_publikacji_w_bpp(self, obj: PublikacjaInstytucji):
        return Rekord.objects.filter(pbn_uid_id=obj.publicationId_id).first()

    def odpowiednik_osoby_w_bpp(self, obj: PublikacjaInstytucji):
        return obj.insPersonId.rekord_w_bpp

    def odpowiednik_instytucji_w_bpp(self, obj: PublikacjaInstytucji):
        return obj.institutionId.rekord_w_bpp



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-17 11:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0085_strony_help_text'),
    ]

    operations = [
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='calkowita_liczba_redaktorow',
            field=models.PositiveIntegerField(blank=True, help_text='Jeżeli dodajesz monografię, wpisz tutaj całkowitą liczbę\n        redaktorów monografii. Ta informacja zostanie użyta w eksporcie \n        danych do PBN. Jeżeli pole to nie zostanie uzupełnione, wartość ta\n        zostanie obliczona i będzie to ilość wszystkich redaktorów \n        przypisanych do danej monografii', null=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='calkowita_liczba_autorow',
            field=models.PositiveIntegerField(blank=True, help_text='Jeżeli dodajesz monografię, wpisz \n        tutaj całkowitą liczbę autorów monografii. Ta informacja zostanie \n        użyta w eksporcie danych do PBN. Jeżeli informacja ta nie zostanie \n        uzupełiona, wartość tego pola zostanie obliczona i będzie to ilość \n        wszystkich autorów przypisanych do danej monografii', null=True),
        ),
    ]
